"""
Convex optimization modeling for cvxopt.

"""

# Copyright (C) 2006-2008 Jacob Mattingley and Stephen Boyd.
#
# This file is part of CVXMOD.
#
# CVXMOD is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
#
# CVXMOD is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.

import cvxopt
cvxopt.printing.options['dformat'] = '%#8.4g'
cvxopt.printing.options['width'] = 14
cvxopt.base.matrix_repr = cvxopt.printing.matrix_str_default

from base import *
from sets import *
from util import *

__all__ = ["matrix", "spmatrix"]
__all__ += base.__all__
__all__ += util.__all__



"""
Convex optimization modeling for cvxopt.

"""

# Copyright (C) 2006-2008 Jacob Mattingley and Stephen Boyd.
#
# This file is part of CVXMOD.
#
# CVXMOD is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
#
# CVXMOD is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.


# Basic atoms to go with jo.py: convex optimization modelling for cvxopt.

# these functions are put here because they have epiorhypos rather than derivatives.

import __builtin__
from cvxmod.base import *
from cvxmod.base import _maxeignum, _mineignum
from cvxmod.setsbase import *
from cvxmod.symbolic import *
from cvxmod.util import iterable

__all__ = ["sum", "min", "max", "norm1", "pos", "neg", "abs", "norminf",
           "maxeig", "mineig"]

class sumfunction(multfunction):
    """The sum of vector elements, or the column sums of a matrix."""
    name = 'sum'
    def __init__(self, arg):
        self.arg = arg
        if equiv(cols(arg), 1) or equiv(rows(arg), 1):
            self.rows = 1
            self.cols = 1
        else:
            self.rows = 1
            self.cols = cols(arg)
        
        if equiv(rows(arg), 1):
            self._lhs = arg
        else:
            self._rhs = arg
    
    def __str__(self):
        return 'sum(%s)' % str(self.arg)

    def _getlhs(self):
        try:
            return self._lhs
        except AttributeError:
            return ones(1, rows(self._rhs), True)
    lhs = property(_getlhs)

    def _getrhs(self):
        try:
            return self._rhs
        except AttributeError:
            return ones(cols(self._lhs), 1, True)
    rhs = property(_getrhs)

    # Next two functions to override multfunction functions.
    def expandtr(self):
        return self

    def getoptvars(self):
        return getoptvars(self.arg)

    def getassertions(self):
        return getassertions(self.arg)

    def getimplicit(self):
        return getimplicit(self.arg)

    def _getconvex(self):
        return isconvex(self.arg)
    convex = property(_getconvex)

    def _getconcave(self):
        return isconcave(self.arg)
    concave = property(_getconcave)

    def _getincreasing(self):
        return isincreasing(self.arg)
    increasing = property(_getincreasing)

    def _getdecreasing(self):
        return isdecreasing(self.arg)
    decreasing = property(_getdecreasing)

    def cvx(self):
        return "sum(%s)" % cvx(self.arg)

    def _getpos(self):
        return ispos(self.arg)
    pos = property(_getpos)

    def _getneg(self):
        return isneg(self.arg)
    neg = property(_getneg)

def sum(obj):
    """Sum of vector elements."""
    # matrix behaviour somewhat undefined.
    # jem. possibly needs revision to reduce behaviour it's overriding. Can't
    # use __builtin__.sum because it does too much.
    if is1x1(obj):
        return obj
    elif isinstance(obj, (matrix, spmatrix)):
        return __builtin__.sum(obj)
    else:
        if isinstance(obj, multfunction) and equiv(obj.lhs, -1):
            # jem experimental. may provide unexpected (and thus unhelpful)
            # simplification at times.
            return -1*sumfunction(-1*obj)
        else:
            return sumfunction(obj)

class absfunction(function, elementwise, convex, positive):
    name = 'abs'
    def __init__(self, arg):
        self.arg = arg
        self.rows = rows(arg)
        self.cols = cols(arg)

    def _getvalue(self):
        return abs(value(self.arg))
    value = property(_getvalue)

    def epiorhypo(self):
        # handles matrices, too, for elementwise. is this a good idea? jem.
        t = dummyvar(size(self.arg))
        s = stdstruct()
        s += (self.arg <= t).split()
        s += (-self.arg <= t).split()
        return (t, s)

def abs(obj):
    """Overloads the built-in abs function for convex optimization."""
    try:
        return __builtin__.abs(obj)
    except TypeError:
        if ispos(obj):
            return obj
        elif isneg(obj):
            return -obj
        else:
            return absfunction(obj)

class singleargmaxfunction(function, convex, increasing):
    """The maximum element of a single argument."""
    name = 'max'
    def __init__(self, arg):
        self.arg = arg
        self.rows = 1
        self.cols = 1

    def getparams(self):
        return getparams(self.arg)

    def epiorhypo(self):
        t = dummyvar(1, 1)
        s = stdstruct()
        s += (self.arg <= ones(size(self.arg), symb=True)*t).split()
        return (t, s)

    def _getvalue(self):
        return max(value(self.arg))
    value = property(_getvalue)

class multiargmaxfunction(function, convex, increasing):
    # jem. the convex here actually doesn't do anything.
    # scalar mutates to appropriate size.
    def __init__(self, args):
        r = 1
        c = 1
        for x in args:
            if is1x1(x) or equiv(size(x), (r, c)) or equiv((r, c), (1, 1)):
                if equiv((r, c), (1, 1)):
                    r = rows(x)
                    c = cols(x)
                    continue
                elif equiv(r, 1) and (equiv(cols(x), 1) or equiv(cols(x), c)):
                    r = rows(x)
                    continue
                elif (equiv(rows(x), 1) or equiv(rows(x), r)) and equiv(c, 1):
                    c = cols(x)
                    continue
            else:
                raise AtomArgsError('incompatibly sized max arguments')

        self.args = args
        self.rows = r
        self.cols = c
        self.name = 'max'

    def getparams(self):
        return getparams(self.args)

    def epiorhypo(self):
        # doesn't handle matrices at the moment.
        t = dummyvar(size(self))
        
        s = stdstruct()
        for x in self.args:
            if equiv(size(x), size(self)):
                s += (x <= ones(size(x))*t).split()
            else:
                # jem: look into whether this is quite what we want?
                s += (x*ones(size(self)) <= ones(size(x))*t).split()

        return (t, s)

    def _getvalue(self):
        return max(*value(self.args))
    value = property(_getvalue)

    def _getnegative(self):
        for x in self.args:
            if not ispos(x):
                return False
        else:
            return True
    negative = property(_getnegative)

    def _getpositive(self):
        for x in self.args:
            if ispos(x):
                return True
        else:
            return False
    positive = property(_getpositive)

    def _getconvex(self):
        for x in self.args:
            if not isconvex(x):
                return False
        else:
            return True
    convex = property(_getconvex)

    def _getincreasing(self):
        for x in self.args:
            if not isincreasing(x):
                return False
        else:
            return True
    increasing = property(_getincreasing)

    def _getdecreasing(self):
        for x in self.args:
            if not isdecreasing(x):
                break
        else:
            return True
    decreasing = property(_getdecreasing)

def max(*args):
    #"""Maximum element of a vector."""
    # need to upgrade this to handle things like max(matrix((1,2)), matrix((2,3))).
    if len(args) > 1:
        if not getoptvars(args) and not getparams(args):
            r = __builtin__.max([rows(x) for x in args])
            c = __builtin__.max([cols(x) for x in args])

            if (r,c) == (1,1):
                return __builtin__.max(args)

            args = list(args)
            for i in xrange(len(args)):
                if size(args[i]) == (1,1):
                    args[i] = args[i]*matrix(ones(r,c))
                elif size(args[i]) != (r,c):
                    raise AtomArgsError('incompatible arguments to max')

            z = zeros(r, c)
            for i in range(r*c):
                z[i] = __builtin__.max([x[i] for x in args])

            return z
        else:
            return multiargmaxfunction(args)
    else:
        arg = args[0]
        if iterable(arg):
            return max(*arg)
        elif isinstance(arg, (int, float)):
            return arg
        elif not getoptvars(arg) and not getparams(arg):
            return __builtin__.max(arg)
        elif is1x1(arg):
            return arg
        else:
            return singleargmaxfunction(arg)
        
class singleargminfunction(function, concave, increasing):
    """The minimum element of a single argument."""
    name = 'min'
    def __init__(self, arg):
        self.arg = arg
        self.rows = 1
        self.cols = 1
    
    def getparams(self):
        return getparams(self.arg)

    def epiorhypo(self):
        t = dummyvar(1, 1)
        s = stdstruct()
        s += (ones(size(self.arg))*t <= self.arg).split()
        return (t, s)

    def _getvalue(self):
        return min(value(self.arg))
    value = property(_getvalue)

class multiargminfunction(function, concave, increasing):
    # jem. the convex here actually doesn't do anything.
    # scalar mutates to appropriate size.
    name = 'min'
    def __init__(self, args):
        r = 1
        c = 1
        for x in args:
            if is1x1(x) or equiv(size(x), (r, c)) or equiv((r, c), (1, 1)):
                if equiv((r, c), (1, 1)):
                    r = rows(x)
                    c = cols(x)
                    continue
                elif equiv(r, 1) and (equiv(cols(x), 1) or equiv(cols(x), c)):
                    r = rows(x)
                    continue
                elif (equiv(rows(x), 1) or equiv(rows(x), r)) and equiv(c, 1):
                    c = cols(x)
                    continue
            else:
                raise AtomArgsError('incompatibly sized min arguments')

        self.args = args
        self.rows = r
        self.cols = c

    def getparams(self):
        return getparams(self.args)

    def epiorhypo(self):
        # doesn't handle matrices at the moment.
        t = dummyvar(size(self))
        
        s = stdstruct()
        for x in self.args:
            if equiv(size(x), size(self)):
                s += (x >= ones(size(x))*t).split()
            else:
                # jem: look into whether this is quite what we want?
                s += (x*ones(size(self)) >= ones(size(x))*t).split()

        return (t, s)

    def _getvalue(self):
        return min(*value(self.args))
    value = property(_getvalue)

    def _getnegative(self):
        for x in self.args:
            if ispos(x):
                return True
        else:
            return False
    negative = property(_getnegative)

    def _getpositive(self):
        for x in self.args:
            if not ispos(x):
                return True
        else:
            return False
    positive = property(_getpositive)

    def _getconcave(self):
        for x in self.args:
            if not isconcave(x):
                return False
        else:
            return True
    concave = property(_getconcave)

    def _getincreasing(self):
        for x in self.args:
            if not isincreasing(x):
                return False
        else:
            return True
    increasing = property(_getincreasing)

    def _getdecreasing(self):
        for x in self.args:
            if not isdecreasing(x):
                break
        else:
            return True
    decreasing = property(_getdecreasing)

def min(*args):
    #"""Maximum element of a vector."""
    # need to upgrade this to handle things like min(matrix((1,2)), matrix((2,3))).
    if len(args) > 1:
        if not getoptvars(args) and not getparams(args):
            r = __builtin__.min([rows(x) for x in args])
            c = __builtin__.min([cols(x) for x in args])

            if (r,c) == (1,1):
                return __builtin__.min(args)

            args = list(args)
            for i in xrange(len(args)):
                if size(args[i]) == (1,1):
                    args[i] = args[i]*matrix(ones(r,c))
                elif size(args[i]) != (r,c):
                    raise AtomArgsError('incompatible arguments to min')

            z = zeros(r, c)
            for i in range(r*c):
                z[i] = __builtin__.min([x[i] for x in args])

            return z
        else:
            return multiargminfunction(args)
    else:
        arg = args[0]
        if iterable(arg):
            return min(*arg)
        elif isinstance(arg, (int, float)):
            return arg
        elif not getoptvars(arg) and not getparams(arg):
            return __builtin__.min(arg)
        elif is1x1(arg):
            return arg
        else:
            return singleargminfunction(arg)
        
class norm1function(function, convex, positive):
    name = 'norm1'
    def __init__(self, arg):
        self.arg = arg
        self.rows = 1
        self.cols = 1

    def _getvalue(self):
        return sum(abs(value(self.arg)))
    value = property(_getvalue)

    def epiorhypo(self):
        t = dummyvar(1, 1)
        (t2, s) = abs(self.arg).epiorhypo()
        s += (t == sum(t2)).split()
        return (t, s)

    def _getinc(obj):
        if ispos(obj.arg): # function is inc.
            return isincreasing(obj.arg)
        elif isneg(obj.arg): # function is dec.
            return isdecreasing(obj.arg)
        else:
            return False
    inc = property(_getinc)

    def _getdec(obj):
        if ispos(obj.arg): # function is inc.
            return isdecreasing(obj.arg)
        elif isneg(obj.arg): # function is dec.
            return isincreasing(obj.arg)
        else:
            return False
    dec = property(_getdec)

    def cvx(self):
        return 'norm(%s, 1)' % str(self.arg)

def norm1(obj):
    if isinstance(obj, (int, float, matrix, spmatrix)):
        return sum(abs(obj))
    else:
        # jem. this (and possibly other situations) will be problematic with
        # params. need some kind of conditional comparison / put on stack
        # for later assertion.
        if cols(obj) > 1:
            raise DimError('norm1 arg must be a column vector')
        else:
            return norm1function(obj)

class norminffunction(function, convex, positive):
    name = 'norminf'
    def __init__(self, arg):
        self.arg = arg
        self.rows = 1
        self.cols = 1

    def _getvalue(self):
        return max(abs(value(self.arg)))
    value = property(_getvalue)

    def epiorhypo(self):
        t = dummyvar(1, 1)
        (abst, s) = abs(self.arg).epiorhypo()
        (maxt, s2) = max(abst).epiorhypo()
        s += s2
        s += (t >= maxt).split()
        return (t, s)

    def _getinc(obj):
        if ispos(obj.arg): # function is inc.
            return isincreasing(obj.arg)
        elif isneg(obj.arg): # function is dec.
            return isdecreasing(obj.arg)
        else:
            return False
    inc = property(_getinc)

    def _getdec(obj):
        if ispos(obj.arg): # function is inc.
            return isdecreasing(obj.arg)
        elif isneg(obj.arg): # function is dec.
            return isincreasing(obj.arg)
        else:
            return False
    dec = property(_getdec)

    def cvx(self):
        return 'norm(%s, inf)' % str(self.arg)

def norminf(obj):
    if isinstance(obj, (int, float, matrix, spmatrix)):
        return max(abs(obj))
    else:
        # jem. this (and possibly other situations) will be problematic with
        # params. need some kind of conditional comparison / put on stack
        # for later assertion.
        if cols(obj) > 1:
            raise DimError('norminf arg must be a column vector')
        else:
            return norminffunction(obj)

class posfunction(function, elementwise, convex, increasing, positive):
    """The positive component of an argument."""
    name ='pos'
    def __init__(self, arg):
        self.arg = arg
        self.rows = rows(arg)
        self.cols = cols(arg)

    def getparams(self):
        return getparams(self.arg)

    def epiorhypo(self):
        # doesn't handle matrices at the moment.
        t = dummyvar(size(self.arg))
        s = stdstruct()
        s += (t >= 0).split()
        s += (t >= self.arg).split()
        return (t, s)

    def _getvalue(self):
        return pos(value(self.arg))
    value = property(_getvalue)

def pos(obj):
    if isinstance(obj, (int, float)):
        return __builtin__.max(obj, 0)
    elif isinstance(obj, (matrix, spmatrix)):
        (r, c) = size(obj)
        z = zeros(r, c, full=True)
        for i in xrange(r*c):
            z[i] = __builtin__.max(obj[i], 0)
        return z
    elif isneg(obj):
        return zeros(size(obj))
    elif ispos(obj):
        return obj
    else:
        return posfunction(obj)

class negfunction(function, elementwise, concave, increasing, negative):
    """The negative component of an argument."""
    name ='neg'
    def __init__(self, arg):
        self.arg = arg
        self.rows = rows(arg)
        self.cols = cols(arg)

    def epiorhypo(self):
        # doesn't handle matrices at the moment.
        t = dummyvar(size(self.arg))
        s = stdstruct()
        s += (t <= 0).split()
        s += (t <= self.arg).split()
        return (t, s)

    def _getvalue(self):
        return neg(value(self.arg))
    value = property(_getvalue)

def neg(obj):
    if isinstance(obj, (int, float)):
        return __builtin__.min(obj, 0)
    elif isinstance(obj, (matrix, spmatrix)):
        (r, c) = size(obj)
        z = zeros(r, c)
        for i in xrange(r*c):
            z[i] = __builtin__.min(obj[i], 0)
        return z
    elif isneg(obj):
        return obj
    elif ispos(obj):
        return zeros(size(obj))
    else:
        return negfunction(obj)

class maxeigfunction(matrixfunction, convex, increasing):
    """The maximum eigenvalue of a matrix."""
    # jem: do know some stuff about positivity / negativity here.
    name ='maxeig'
    def __init__(self, arg):
        if not equiv(rows(arg), cols(arg)):
            raise DimError("cannot find maximum eigenvalue of non-square object")
        self.arg = arg
        self.rows = 1
        self.cols = 1

    def epiorhypo(self):
        # doesn't handle matrices at the moment.
        t = dummyvar()
        s = stdstruct()
        if not issymm(self.arg):
            xs = dummyvar(size(self.arg), symm=True)
            s += (xs == self.arg).split()
            s += (xs |lt| t*eye(rows(self.arg), True)).split()
        else:
            s += (self.arg |lt| t*eye(rows(self.arg), True)).split()
        return (t, s)

    def _getvalue(self):
        return maxeig(value(self.arg))
    value = property(_getvalue)
    def cvx(self):
        return "lambda_max(%s)" % str(self.arg)

def maxeig(obj):
    if isinstance(obj, (int, float, matrix, spmatrix)):
        return _maxeignum(obj)
    else:
        return maxeigfunction(obj)

class mineigfunction(matrixfunction, concave, increasing):
    """The minimum eigenvalue of a matrix."""
    # jem: do know some stuff about positivity / negativity here.
    name ='mineig'
    def __init__(self, arg):
        if not equiv(rows(arg), cols(arg)):
            raise DimError("cannot find minimum eigenvalue of non-square object")
        self.arg = arg
        self.rows = 1
        self.cols = 1

    def epiorhypo(self):
        # doesn't handle matrices at the moment.
        t = dummyvar()
        s = stdstruct()
        if not issymm(self.arg):
            xs = dummyvar(size(self.arg), symm=True)
            s += (xs == self.arg).split()
            s += (xs |gt| t*eye(rows(self.arg), True)).split()
        else:
            s += (self.arg |gt| t*eye(rows(self.arg), True)).split()
        return (t, s)

    def _getvalue(self):
        return mineig(value(self.arg))
    value = property(_getvalue)

    def cvx(self):
        return "lambda_min(%s)" % str(self.arg)

def mineig(obj):
    if isinstance(obj, (int, float, matrix, spmatrix)):
        return _mineignum(obj)
    else:
        return mineigfunction(obj)



""" This file defines a set of functions and classes for cycle analysis
    according to
    [1] RR Vallabhajosyula, V Chickarmane, HM Sauro: Conservation analysis of
        large biochemical networks. Bioinformatics 22(3): 346-353 (2006)
"""

from defines import roundAndCut, makeCoefNameTerm, displayPermutedMatrix
from numpy import array, zeros, empty, eye, nonzero, vectorize
try:
    import pygsl, pygsl.permutation, pygsl.linalg
    _pygsl_avail = True
except ImportError:
    _pygsl_avail = False


_myround = lambda x : roundAndCut(x, 8)
_vmyround = vectorize(_myround)


class FluxCycle(object):
    """ class for representing flux cycles

    Member variables:

    lhs     - matrix
    rhs     - vector or list of strings
    indices - array of indices linking the columns of lhs to the full list of
              reactions (indices[i] is the index of column i in the original
              list

    A flux cycle is represented by a linear system of equations, which is stored
    in the variables lhs and rhs for the left-hand and right-hand sides.
    """

    def __init__(self, lhs=None, rhs=None, indices=[]):
        """ construct a new FluxCycle object
        """
        self.lhs = lhs
        self.rhs = rhs
        self.indices = indices


    def display(self, reactionNames):
        """ return a list of lines for displaying the linear system of equations
        """
        if self.lhs is None or self.rhs is None:
            return []

        # First convert left-hand side to (coefficient, name) pairs joined by
        # plus or minus signs
        lines = displayPermutedMatrix(self.lhs, self.indices, reactionNames)

        try:
            float(self.rhs[0])
            isString = False
        except ValueError:
            isString = True
        except IndexError:
            return lines

        # Pad lines to same length and add right-hand side
        maxlen = len(max(lines, key=len))
        nRows = len(self.lhs)
        if isString:
            for i in range(nRows):
                if self.rhs[i] == "":
                    lines[i] = lines[i].ljust(maxlen) + " = 0"
                else:
                    lines[i] = "%s = %s" % (lines[i].ljust(maxlen), self.rhs[i])
        else:
            for i in range(nRows):
                lines[i] = "%s = %g" % (lines[i].ljust(maxlen), self.rhs[i])

        return lines


def gaussJordan(M, epsilon = 1E-8):
    """ transform the given matrix to reduced row-echelon form using Gauss-
        Jordan elimination and read off the rank of M. The operation is
        performed in place, i.e. M is overwritten with the result.
    """
    lead = 0
    nRows, nCols = M.shape
    rank = nRows  # assume full rank

    for i in range(nRows):
        # Find leading coefficient in row i
        while lead < nCols and abs(M[i, lead]) < epsilon:
            # Look for non-zero pivot element in other rows
            j = i+1
            while j < nRows and abs(M[j, lead]) < epsilon:
                j += 1

            if j < nRows:
                # Swap row j with row i
                temp = M[i, :].copy()
                M[i, :] = M[j, :]
                M[j, :] = temp
                break

            #  If all elements in column are zero, proceed to next column
            lead += 1

        if lead >= nCols:
            rank = i  # rank is index of first all-zero row
            break

        M[i, :] /= M[i, lead]  # Divide row i by leading coefficient

        # Subtract scaled row i from all rows below and above to set all
        # M[j, lead] to 0
        for j in range(i):
            M[j, :] -= M[j, lead] * M[i, :]
        for j in range(i+1, nRows):
            M[j, :] -= M[j, lead] * M[i, :]

        lead += 1

    return rank


def ref2rref(M, epsilon = 1E-8):
    """ convert a matrix M that is already in row-echelon form to reduced
        row-echelon form and read off the rank of M. The operation is performed
        in place, i.e. M is overwritten with the result.
    """
    lead = 0
    nRows, nCols = M.shape
    rank = nRows  # assume full rank

    for i in range(nRows):
        # Find leading coefficient in row i
        while lead < nCols and abs(M[i, lead]) < epsilon:
            lead += 1

        if lead >= nCols:
            rank = i  # rank is index of first all-zero row
            break

        M[i, :] /= M[i, lead]  # Divide row i by leading coefficient

        # Subtract scaled row i from all rows above to set all M[j, lead] to 0
        for j in range(i):
            M[j, :] -= M[j, lead] * M[i, :]

        lead += 1

    return rank


def qrpt_decomp2(A):
    """ compute the QR factorization with column pivoting of the matrix A
        (wrapper for the corresponding GSL function)

    Keyword arguments:

    A     -- input matrix

    Returns tuple (p, q, r) with

    p     -- permutation P as list (column index in R -> original index in A)
    q     -- orthogonal matrix Q
    r     -- upper triangular matrix R with lower (all-zero) part removed

    with  QRP' = A
    """
    m, n = A.shape
    dtype = A.dtype
    q = zeros((m, m), dtype)
    r = zeros((m,n), dtype)
    tau = pygsl.permutation.Permutation(min(m,n))
    p = pygsl.permutation.Permutation(n)
    sig = pygsl.permutation.Permutation(n)

    # Perform QR decomposition with column pivoting on matrix A
    pygsl.linalg._gslwrap.gsl_linalg_QRPT_decomp2(A, q, r, tau, p, sig)

    # Transform R to reduced row-echelon form and read off rank
    rank = ref2rref(r)

    return p.toarray(), q, r[:rank, :]


class CycleAnalyzer(object):
    """ class for cycle analysis based on [1]

    Member variables:

    gamma       -- matrix of linear dependencies as defined in [1]
    gIndices    -- indices of columns of gamma in original stoichiometric matrix
    r           -- reduced row-echelon form of permuted stoichiometric matrix
    rIndices    -- indices of columns of r in original stoichiometric matrix
    nCycleReacs -- number of reactions that are involved in at least one cycle
                   rIndices[:nCycleReacs] are the indices of these
    """

    def __init__(self):
        """ construct a new CycleAnalyzer object
        """
        self.gamma = None
        self.gIndices = []
        self.nCycleReacs = 0
        self.r = None
        self.rIndices = []


    @staticmethod
    def computeGamma(S):
        """ perform cycle analysis on matrix S via QR decomposition with column
            pivoting as developed in [1]
        """
        p, _, r = qrpt_decomp2(S)
        n = S.shape[1]     # number of columns in S = number of reactions
        rank = r.shape[0]  # number of rows of R = rank of S
        nRowsG = n-rank
        g = empty((nRowsG, n), S.dtype)
        g[:, :rank] = -r[:rank, rank:].T
        g[:, rank:] = eye(nRowsG)
        return p, r, g


    def getDependent(self):
        """ return list of indices of dependent reactions
        """
        if self.gamma is None:
            return []

        return self.gIndices[-len(self.gamma):]


    def displayGamma(self, reactionNames):
        """ return a list of lines for displaying gamma as linear combinations
        """
        if self.gamma is None:
            return []
        return displayPermutedMatrix(self.gamma, self.gIndices, reactionNames)


    def computeCycles(self, model, internalOnly=True):
        """ perform cycle analysis on the given MetabolicModel

        Results are stored in member variables gamma, gIndices, r, rIndices.
        Afterwards, cycle descriptions can be requested by call of
        getCycleDescriptions().
        """
        if not _pygsl_avail:
            raise ImportError("pygsl not found")

        S = array(model.getStoichiometricMatrix())
        nCols = S.shape[1]
        if internalOnly:
            internalReactions = model.getInternalFluxes()
            submodel = model.getSubmodelByList(internalReactions)
            S_ = array(submodel.getStoichiometricMatrix())
            nColsG = len(submodel)
        else:
            S_ = S
            nColsG = len(model)

        self.gIndices, _, self.gamma = self.computeGamma(S_)

        if internalOnly:
            # Replace indices in submodel with original indices
            for i in range(nColsG):
                self.gIndices[i] = model.reactionDict[submodel.reactions[
                                                        self.gIndices[i]].name]

        # Round Gamma matrix to 8 digits and truncate near-zero values
        self.gamma = _vmyround(self.gamma)

        # Perform Gauss-Jordan elimination of full matrix (without column
        # pivoting) to get relations between fluxes for all reactions
        # participating in cycles

        # First create permutation of columns of S such that
        # - left-most are the dependent reactions
        # - to the right follow all other reactions participating in any cycle
        # - right-most are the reactions not participating in any cycle
        self.rIndices = empty(nCols, 'i')

        # First store indices of dependent reactions
        nDep = len(self.gamma)
        dependent = self.gIndices[-nDep:]
        self.rIndices[:nDep] = dependent
        # Create mask for column indices already covered by rIndices
        got = zeros(nCols, bool)
        got[dependent] = True

        # Next store indices of independent reactions participating in cycles
        i = nDep
        for j in nonzero(self.gamma[:, :-nDep])[1]:
            index = self.gIndices[j]
            if not got[index]:
                self.rIndices[i] = index
                got[index] = True
                i += 1
        self.nCycleReacs = i

        # Finally add all other reactions
        for index in range(nCols):
            if not got[index]:
                self.rIndices[i] = index
                got[index] = True
                i += 1

        # Permute stoichiometric matrix and perform Gauss-Jordan elimination
        self.r = S[:, self.rIndices]
        rank = gaussJordan(self.r)
        # Remove all-zero rows from R
        self.r = self.r[:rank, :]

        # Also round r matrix to 8 digits and truncate near-zero values
        self.r = _vmyround(self.r)


    def getCycleDescriptions(self, solution, sIndices=None):
        """ generate descriptions of each cycle from Gamma and R, depending on
            the flux distribution given in solution

        Keyword arguments:

        solution -- an FBA solution vector or list of reaction names
        sIndices -- sIndices[i] is the index in the original model corresponding
                    to solution[i] - if not given, assumed as [0..len(solution)]

        Returns: list of FluxCycle objects
        """
        try:
            float(solution[0])
            isString = False
        except ValueError:
            isString = True
        except IndexError:
            return []

        if sIndices == None:
            sIndices = range(len(solution))

        dependent = set(self.getDependent())
        dtype = self.r.dtype
        nColsR = self.r.shape[1]
        cycles = []

        # Build reverse permutations from rIndices and sIndices
        rIndicesRev = empty(nColsR, 'i')
        for i in range(nColsR):
            rIndicesRev[self.rIndices[i]] = i
        sIndicesRev = {}
        for i in range(len(sIndices)):
            sIndicesRev[sIndices[i]] = i

        # Set of all reactions participating in at least one cycle
        cycleIndices = self.rIndices[:self.nCycleReacs]
        allCycleReacs = set(cycleIndices)
#        print "allCycleReacs", allCycleReacs

        # Each row in Gamma corresponds to one cycle
        for rowG in self.gamma:
            cycle = FluxCycle()
            cycle.indices = cycleIndices

            # Only reactions with non-zero coefficient are part of the cycle
            nz = nonzero(rowG)[0]
            nNonzero = len(nz)
            cycle.lhs = empty((nNonzero-1, self.nCycleReacs), dtype)
            if isString:
                cycle.rhs = [""]*(nNonzero-1)
            else:
                cycle.rhs = zeros(nNonzero-1, dtype)
            setCycIndices = set(self.gIndices[nz])

            # Build left-hand side of lin. eq. system from selected columns of R
            # Build right-hand side from masked columns and solution
            i = 0
#            k = 0
#            print "begin Cycle"
#            print " indices", cycle.indices
#            print " processing R"
            for rowR in self.r:
#                k += 1
#                print " row %u" % k

                # Only look at non-zero entries of rowR
                nzR = nonzero(rowR)[0]
                rIndicRed = self.rIndices[nzR]
                setRIndicRed = set(rIndicRed)

                # Skip a) rows of R not pertaining to the reactions of the cycle
                #         (i.e. those with all zeros in all selected columns)
                #  and b) rows of R pertaining to more than one cycle
                if (len(setCycIndices & setRIndicRed) == 0 or
                    len((setRIndicRed | setCycIndices) & dependent) > 1):
                    continue

#                print rowR
#                print "  match! indices:", rIndicRed
#                print "  values:", rowR[nzR]
#                sValues = []
#                for j in rIndicRed:
#                    if j in sIndicesRev:
#                        sValues.append(solution[sIndicesRev[j]])
#                    else:
#                        sValues.append(nan)
#                print "  flux values:", sValues
                cycle.lhs[i, :] = rowR[:self.nCycleReacs]
#                print "row in cycle desc.:"
#                print cycle.lhs[i, :]

                if isString:
                    for j in rIndicRed:
                        if j not in allCycleReacs:
                            cycle.rhs[i] += makeCoefNameTerm(cycle.rhs[i],
                                -rowR[rIndicesRev[j]], solution[sIndicesRev[j]])
                else:
                    for j in rIndicRed:
                        if j not in allCycleReacs:
                            cycle.rhs[i] -= (rowR[rIndicesRev[j]] *
                                             solution[sIndicesRev[j]])
#                print "rhs:", cycle.rhs[i]

                i += 1
                if i >= nNonzero-1:
                    break

            cycles.append(cycle)

        return cycles



"""
This module defines the class MomaProblem, which represents a quadratic problem
to be solved in MOMA analysis (Maximization of Metabolic Adjustment).

The MomaProblem is a quadratic optimization problem with objective function
    Z = (v-v_wt)'*(v-v_wt) = v'*v - 2*v_wt'*v + v_wt'*v_wt,
i.e. the goal is to minimize the quadratic distance to the FBA solution for the
wildtype.

OpenOpt demands a formulation in terms of a matrix H and a vector f:
    Z_oo = 0.5 v'*H*v + f'*v

For H = Identity and f = -v_wt, minimization of Z_oo is equivalent to
minimization of Z, because the constant v_wt'*v_wt can be omitted and the
function can be scaled by a factor.

If both CVXMOD and CVXPY are installed, the CVXMOD interface will be used
preferably, as it is faster. Only if CVXMOD fails to converge, CVXPY is used.


This file is part of metano.
Copyright (C) 2010-2014 Alexander Riemer,
Braunschweig University of Technology,
Dept. of Bioinformatics and Biochemistry

metano is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

metano is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with metano.  If not, see <http://www.gnu.org/licenses/>.
"""

# TODO Release:
# - remove 'weights'

from numpy import array, eye, inf, isnan, diag, sqrt, linalg
from defines import SolverStatus, cvxToSolverStatus, cvxpyToSolverStatus
from linearproblem import SolverError
from quadraticproblem import QuadraticProblem, _cvxmod_avail, _cvxpy_avail
if _cvxmod_avail:
    import cvxmod
if _cvxpy_avail:
    import cvxpy

#print cvxpy.Problem.__dict__.keys()


class MomaProblem(QuadraticProblem):

    matrix_tol = 1E-5     # tolerance for matrix constraints
    bounds_tol = 1E-6     # tolerance for bounds constraints
    BIG_NUMBER = 100000


    def __init__(self, Aeq, beq=None, Aineq=None, bineq=None, lb=None, ub=None,
                 solver=QuadraticProblem.solvers['default'], solution=None,
                 weights=None):
        """ initialize the quadratic problem

        Keyword arguments:

        Aeq         -- matrix of equality constrains (Aeq * v = beq)
        beq         -- right-hand side vector of equality constraints
        Aineq       -- matrix of inequality constrains (Aineq * v <= bineq)
        bineq       -- right-hand side vector of inequality constraints
        lb          -- list of lower bounds (indexed like matrix columns)
        ub          -- list of upper bounds (indexed like matrix columns)
        solver      -- QP or NLP solver to be used for optimization
        solution    -- the solution for the wildtype
        weights     -- weight vector
        """
        print "Momaproblem", solver
        QuadraticProblem.__init__(self, Aeq, beq, Aineq, bineq, lb, ub, solver)
        if weights is None:
            self.weights = None
            self.obj.H = eye(len(lb))
        else:
            self.weights = array(weights)
            if len(weights) != len(lb):
                raise ValueError("Error: Weight vector has wrong length")
            self.obj.H = diag(self.weights)

        if solution is not None:
            self.obj.f = -array(solution)


    def setWtSolution(self, solution):
        """ set the solution for the wildtype against which to perform MOMA in
            the objective function definition of the underlying QuadraticProblem
        """
        self.obj.f = -array(solution)


    def minimize(self, **kwargs):
        if self.solver.startswith("cvx"):
            if not _cvxmod_avail and not _cvxpy_avail:
                raise SolverError("Solver '%s' not available. See installation "
                                  "instructions on http://metano.tu-bs.de" %
                                  self.solver)
            nCols = len(self.lb)
            lb = [-self.BIG_NUMBER]*nCols
            ub = [ self.BIG_NUMBER]*nCols
            for i in range(nCols):
                if self.lb[i] > -inf:
                    lb[i] = self.lb[i]-self.bounds_tol
                if self.ub[i] < inf:
                    ub[i] = self.ub[i]+self.bounds_tol

            doCvxPy = self.solver == "cvxpy_only" or not _cvxmod_avail
            if _cvxmod_avail and (self.solver == "cvxopt_qp" or
                                  self.solver == "cvxmod_only"):
                self._cvxmod_minimize(lb, ub, **kwargs)
                if self.status == SolverStatus.UNKNOWN:
                    doCvxPy = True

            if doCvxPy and _cvxpy_avail:
                self._cvxpy_minimize(lb, ub, **kwargs)

            return self.obj_value, self.solution
        else:  # use OpenOpt solver
            if self.weights is not None:
                self.obj.f *= self.weights
            solution = QuadraticProblem.minimize(self, **kwargs)
            if self.weights is not None:
                self.obj.f /= self.weights
            return solution


    def _cvxmod_minimize(self, lb, ub, **kwargs):
        """ solve quadratic problem using CVXMOD interface

        Keyword arguments:

        lb, ub  -- vectors of lower and upper bounds (potentially modified by
                   added tolerance)

        Modified member variables:

        status, solution, obj_value

        Returns: nothing
        """
        # shorter names
        v = self.cvxmodV
        A = self.cvxmodMatrix
        minus_w = cvxmod.matrix(self.obj.f)  # negative wildtype solution
        lb = cvxmod.matrix(lb)
        ub = cvxmod.matrix(ub)
        if self.weights is not None:
            weights = cvxmod.matrix(diag(sqrt(self.weights)))
        if self.Aineq is not None:
            Aineq = cvxmod.matrix(array(self.Aineq))
            bineq = cvxmod.matrix(self.bineq)
            if self.weights is None:
                p = cvxmod.problem(cvxmod.minimize(cvxmod.norm2(v+minus_w)),
                            [cvxmod.abs(A*v) < self.matrix_tol,
                             Aineq*v <= bineq+self.matrix_tol,
                             v >= lb, v <= ub])
            else:
                p = cvxmod.problem(cvxmod.minimize(cvxmod.norm2(weights*
                            (v+minus_w))), [cvxmod.abs(A*v) < self.matrix_tol,
                             Aineq*v <= bineq+self.matrix_tol,
                             v >= lb, v <= ub])
        else:
            if self.weights is None:
                p = cvxmod.problem(cvxmod.minimize(cvxmod.norm2(v+minus_w)),
                            [cvxmod.abs(A*v) < self.matrix_tol,
                             v >= lb, v <= ub])
            else:
                p = cvxmod.problem(cvxmod.minimize(cvxmod.norm2(weights*
                            (v+minus_w))), [cvxmod.abs(A*v) < self.matrix_tol,
                             v >= lb, v <= ub])

        self.status = cvxToSolverStatus(p.solve())

        if not v.value:
            self.solution = []
        else:
            self.solution = array(list(v.value))
        try:
            self.obj_value = p.value
        except cvxmod.OptvarValueError:
            self.obj_value = inf


    def _cvxpy_minimize(self, lb, ub, **kwargs):
        """ solve quadratic problem using CVXPY interface

        Keyword arguments:

        lb, ub  -- vectors of lower and upper bounds (potentially modified by
                   added tolerance)

        Modified member variables:

        status, solution, obj_value

        Returns: nothing
        """
        # shorter names
        v = self.cvxpyV
        A = self.cvxpyMatrix
        minus_w = array(self.obj.f).T  # negative wildtype solution
        lb = array(lb).T
        ub = array(ub).T
        if self.weights is not None:
            weights = array(diag(sqrt(self.weights)))
        if self.Aineq is not None:
            Aineq = array(self.Aineq)
            bineq = array(self.bineq)
            p = cvxpy.Problem(cvxpy.Minimize(cvxpy.norm2(v+minus_w)),
                        [cvxpy.abs(A*v) <= self.matrix_tol,
                         (Aineq*v, bineq) <= v, v >= lb,
                         v <= ub])
        else:
            p = cvxpy.Problem(cvxpy.Minimize(cvxpy.norm2(v+minus_w)),
                        [cvxpy.abs(A*v) <= self.matrix_tol,
                         v >= lb, v <= ub])

        # Configure program ('abstol', 'feastol', 'reltol', 'maxiters')
        """for k in kwargs:
            try:
                p.options[k] = kwargs[k]
            except KeyError:
                continue"""
        print kwargs
        self.obj_value = p.solve(solver="CVXOPT", reltol = kwargs['reltol'], 
            abstol = kwargs['abstol'], feastol = kwargs['feastol'])#, 
            #maxiters = kwargs['maxiters'], gtol = kwargs['gtol'], maxIter = kwargs['maxIter'])
        self.status = cvxpyToSolverStatus(self.obj_value)

        if len(v.value) == 0 or isnan(v.value[0,0]):
            self.solution = []
        else:
            self.solution = array(v.value.T)[0,:]


    def eval(self, v):
        """ evaluate objective function for vector v
        """
        if self.weights is None:
            return linalg.norm(array(v)+self.obj.f)
        else:
            return linalg.norm(diag(sqrt(self.weights))*(array(v)+self.obj.f))



"""
This module defines the class QuadraticProblem, which represents a quadratic
problem to be solved by an appropriate solver.


This file is part of metano.
Copyright (C) 2010-2014 Alexander Riemer,
Braunschweig University of Technology,
Dept. of Bioinformatics and Biochemistry

metano is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

metano is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with metano.  If not, see <http://www.gnu.org/licenses/>.
"""

from openopt import QP
from numpy import array
from linearproblem import LinearProblem

try:
    from cvxmod import matrix as cvxmod_matrix, optvar
    _cvxmod_avail = True
except ImportError:
    _cvxmod_avail = False
try:
    from cvxpy import *
    _cvxpy_avail = True
except ImportError:
    _cvxpy_avail = False


class QuadraticProblem(LinearProblem):
    # Table of quadratic problem solvers with corresponding OpenOpt name
    solvers = {'default'  : 'cvxopt_qp',          # cvxopt = try cvxmod first
               'cvxopt'   : 'cvxopt_qp',          #  (fast) - if no convergence, 
               'cvxopt_qp': 'cvxopt_qp',          #           try cvxpy (slower)
               'cvxmod'   : 'cvxmod_only',
               'cvxpy'    : 'cvxpy_only',
               # Or use an NLP solver:
               'ralg'     : 'nlp:ralg',           # for medium-sized problems;
                                                  #   ill-conditioned
               'cobyla'   : 'nlp:scipy_cobyla',   # recommended (for NLP)
               'algencan' : 'nlp:algencan',       # recommended (for NLP)
               'slsqp'    : 'nlp:scipy_slsqp',
               'ipopt'    : 'nlp:ipopt',
               'lincher'  : 'nlp:lincher'}        # very primitive
    
    @staticmethod
    def isCvxQpSolver(solver):
        return solver.startswith("cvx") or solver == "default"


    class OoQp(object):
        """ class for objective function in OpenOpt's QP format:

        Z(v) = 0.5*v'*H*v + f*v with

            H - matrix of coefficients of quadratic terms and
            f - vector of coefficients of linear terms
        """
        def __init__(self, H=None, f=None):
            self.H = H
            self.f = f


    def __init__(self, Aeq, beq=None, Aineq=None, bineq=None, lb=None, ub=None,
                 solver=solvers['default']):
        """ initialize the quadratic problem

        Keyword arguments:

        Aeq     -- matrix of equality constrains (Aeq * v = beq)
        beq     -- right-hand side vector of equality constraints
        Aineq   -- matrix of inequality constrains (Aineq * v <= bineq)
        bineq   -- right-hand side vector of inequality constraints
        lb      -- list of lower bounds (indexed like matrix columns)
        ub      -- list of upper bounds (indexed like matrix columns)
        solver  -- solver to be used for Quadratic Programming
        """
        LinearProblem.__init__(self, Aeq, beq, Aineq, bineq, lb, ub, solver)
    
        self.obj = QuadraticProblem.OoQp()
        if _cvxmod_avail:
            self.cvxmodMatrix = cvxmod_matrix(array(Aeq))
            self.cvxmodV = optvar('v', self.cvxmodMatrix.size[1], 1)
        if _cvxpy_avail:
            self.cvxpyMatrix = array(Aeq)
            self.cvxpyV = Variable(self.cvxpyMatrix.shape[1], 1, name='v')


    def setObjective(self, quad_coef, lin_coef):
        """ set the objective function in OpenOpt's QP format:

        Z(v) = 0.5*v'*H*v + f*v with

            H = quad_coef, matrix of coefficients of quadratic terms and
            f = lin_coef, vector of coefficients of linear terms
        """
        self.obj = QuadraticProblem.OoQp(quad_coef, lin_coef)

    def resetObjective(self):
        """ shadow LinearProblem.resetObjective """
        pass


    def minimize(self, **kwargs):
        """ solve the quadratic problem using OpenOpt

        Returns:
        obj_value, solution

        obj_value -- value of the objective function at the discovered solution
        solution  -- the solution flux vector (indexed like matrix columns)
        """
        qp = QP(self.obj.H, self.obj.f, A=self.Aineq, Aeq=self.Aeq,
                b=self.bineq, beq=self.beq, lb=self.lb, ub=self.ub, **kwargs)
        qp.debug=1
        r = qp.solve(self.solver)

        if r.istop <= 0 or r.ff != r.ff:  # check halting condition
            self.obj_value = 0
            self.solution = []
            self.istop = r.istop
        else:
            self.obj_value = r.ff
            self.solution = r.xf
            self.istop = r.istop

        return self.obj_value, self.solution



#!/usr/bin/env python

""" metano Minimization of Metabolic Adjustment (MOMA)

This script performs Minimization of Metabolic Adjustment (MOMA) on the
metabolic network (given as a set of reaction equations) and inequality
constraints of a mutant organism against the given FBA solution for the
corresponding wildtype.

It parses a reaction file and a scenario file, formulates the corresponding
linear, quadratic, or other nonlinear optimization problem, and calls an
appropriate solver.


This file is part of metano.
Copyright (C) 2010-2014 Alexander Riemer,
Braunschweig University of Technology,
Dept. of Bioinformatics and Biochemistry

metano is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

metano is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with metano.  If not, see <http://www.gnu.org/licenses/>.
"""

# TODO Release:
# - remove wMOMA: _DEFAULT_ALPHA/BETA, weights, weightsRed, weighted
# - remove option_group("Extra options for weighted MOMA")
# - remove step 7. (Optionally) Read FVA solution and compute weights

from fba import OptionParser, FbAnalyzer
from reactionparser import ReactionParser
from paramparser import ParamParser
from metabolicmodel import MetabolicModel
from metabolicflux import MetabolicFlux
from momaproblem import MomaProblem
from nonlinearproblem import NonLinearProblem
from startpointiterator import StartPointIterator
from fva import FvAnalyzer
from defines import (SolverStatus, printHistogram, printStatus, FbaParam,
                     COPYRIGHT_VERSION_STRING)
from numpy import array, dot, nan, sqrt, ndarray
from math import exp
import os

_DEFAULT_ALPHA    = 1E-6
_DEFAULT_BETA     = .15


class MomaAnalyzer(object):
    """ Class for Minimization of Metabolic Adjustment (MOMA)
    """

    def __init__(self, solver="default"):
        self.solver = solver

    @staticmethod
    def getWeightsFromFluxVar(model, fvaMinmax, alpha=_DEFAULT_ALPHA,
                              beta=_DEFAULT_BETA):
        """ compute weights for wMOMA from the given FVA solution

        Keyword arguments:

        model        -- the MetabolicModel
        fvaMinmax    -- dict { reaction : flux minimum, flux maximum }
        alpha, beta  -- parameters for weights computation:
                        w = alpha + exp(-beta*(fvaMax-fvaMin))

        Returns vector of weights (list indexed like model's reactions list)
        """
        weights = []
        for rea in model.getReactionNames():
            mini, maxi = fvaMinmax[rea]
            weights.append(alpha+exp(beta*(mini-maxi)))
        return weights


    def run(self, matrix, lb, ub, wtSolution, eqs=[], ineqs=[], numIter=1,
            weights=None):
        """ construct and solve the quadratic optimization problem

        Keyword arguments:

        matrix     -- the stoichiometric matrix of the metabolic network
        lb         -- list of lower bounds (indexed like matrix columns)
        ub         -- list of upper bounds (indexed like matrix columns)
        wtSolution -- FBA solution for the wildtype
                     (list indexed like matrix columns)
        eqs        -- list of (coefficient vector, right-hand side) pairs for
                      additional equality constraints
        ineqs      -- list of - '' - for additional inequality constraints
        numIter    -- number of iterations of NLP to perform
        weights    -- weight vector for weighted MOMA (None -> perform regular
                      MOMA, else: weight flux i with weights[i])

        Returns:
        distance, solution, status

        distance   -- minimum possible distance from wtSolution with the given
                      matrix & constraints
        solution   -- a flux vector with minimal distance to wtSolution
                     (list indexed like matrix columns)
        status     -- SolverStatus after optimization
        """
        wtVec = array(wtSolution)
        # Construct matrices of original equality and inequality constraints
        Aeq, beq, Aineq, bineq = FbAnalyzer.makeConstraintMatrices(matrix, eqs,
                                                                   ineqs)
        try:
            if MomaProblem.isCvxQpSolver(self.solver):
                ps = MomaProblem(Aeq, beq, Aineq, bineq, lb, ub, self.solver,
                                 wtVec, weights)
            else:
                ps = NonLinearProblem(Aeq, beq, Aineq, bineq, lb, ub,
                                      self.solver)
                if weights is None:
                    ps.setObjective(lambda x : dot(x-wtVec, x-wtVec))
                    ps.setObjGrad(lambda x : 2*(x-wtVec))
                else:
                    if not isinstance(weights, ndarray):
                        weights=array(weights)
                    ps.setObjective(lambda x : dot(x-wtVec, (x-wtVec)*weights))
                    ps.setObjGrad(lambda x : 2*sqrt(weights)*(x-wtVec))
                spIter = StartPointIterator(len(lb), numIter)
                spIter.setRange(-1., 1.)
                ps.setStartPointIterator(spIter)

        except ValueError, strerror:
            print strerror
            exit()

        try:
            distance, solution = ps.minimize(gtol=1e-5, maxIter=200,
                                             maxFunEvals=1e5, abstol=1e-6,
                                             feastol=1e-5, reltol=1e-5,
                                             maxiters=100)
        except ValueError, strerror:
            if self.solver == "default":
                print "Default solver reported an error:"
            else:
                print "Solver %s reported an error:" % self.solver
            print strerror
            print "Consider trying a different solver."
            return nan, [], ps.status
        except KeyError, strerror:
            print strerror
            exit()

        if hasattr(ps, "getHistogram"):
            printHistogram(*ps.getHistogram(32))

        return distance, solution, ps.status


    def runOnModel(self, model, wtSolution, linConstraints=[], numIter=1,
                   weights=None, blockedReactions=[]):
        """ construct and solve the quadratic optimization problem
          - this function runs directly on a MetabolicModel and a MetabolicFlux

        Keyword arguments:

        model          -- the MetabolicModel
        wtSolution     -- FBA solution for the wildtype (given as MetabolicFlux)
        linConstraints -- list of LinearConstraint objects
        numIter        -- number of iterations of NLP to perform
        weights        -- weight vector for weighted MOMA (None -> perform
                          regular MOMA, else: weight flux i with weights[i])
        blockedReactions
                       -- remove the given blocked reactions before analysis
                          (faster and gives an optimal solution, as well)

        Returns:
        distance, solution, status, dimReduced

        distance   -- minimum possible distance from wtSolution with the given
                      matrix & constraints
        solution   -- a solution with minimal distance to wtSolution
                     (as MetabolicFlux)
        status     -- SolverStatus after optimization
        dimReduced -- pair (nRows, nColumns) with dimensions of reduced matrix
        """
        if blockedReactions:
            modelRed = model.getSubModelByExcludeList(blockedReactions)
            cbz = model.canBeZero(blockedReactions)
            nonZeroDeadEnds = [blockedReactions[i] for i in
                               range(len(blockedReactions)) if not cbz[i]]
            if nonZeroDeadEnds:
                print ("The following blocked reactions are constrained to a "
                       "non-zero flux:\n  " + "\n  ".join(nonZeroDeadEnds) +
                       "\nThe problem is infeasible.")
                return (nan, MetabolicFlux(), SolverStatus.PRIM_INFEAS,
                        array(modelRed.getStoichiometricMatrix()).shape)

            reactionsRed = set(modelRed.getReactionNames())
            if weights is None:
                weightsRed = None
            else:
                weightsRed = [0.]*len(modelRed)
                for rea in wtSolution:
                    if rea in reactionsRed:
                        weightsRed[modelRed.reactionDict[rea]] = \
                            weights[model.reactionDict[rea]]
                    weightsRed = array(weightsRed)
        else:
            modelRed = model
            weightsRed = weights

        matrix = array(modelRed.getStoichiometricMatrix())
        dimReduced = matrix.shape
        lb, ub = map(array, modelRed.getBounds())
        try:
            eqs, ineqs = ParamParser.linConstraintsToVectors(linConstraints,
                                                          modelRed.reactionDict)
        except ValueError:
            # If any linear constraint is contradictory, return empty solution
            return (nan, MetabolicFlux(),SolverStatus.PRIM_INFEAS,
                    array(modelRed.getStoichiometricMatrix()).shape)

        distance, solution, status = self.run(matrix, lb, ub,
                                      wtSolution.getVecOrderedByModel(modelRed),
                                      eqs, ineqs, numIter, weightsRed)
        flux = MetabolicFlux(modelRed, solution)

        # Add removed reactions with flux 0. and original bounds to solution
        if len(modelRed) != len(model) and solution != []:
            reactionsRed = set(modelRed.getReactionNames())
            for rea in model:
                if rea.name not in reactionsRed:
                    flux.fluxDict[rea.name] = 0.
                    flux.boundsDict[rea.name] = (rea.lb, rea.ub)

        return distance, flux, status, dimReduced


    def evalObjFunc(self, wtVec, solution, weights=None):
        if MomaProblem.isCvxQpSolver(self.solver):
            ps = MomaProblem([[0.]*len(wtVec)], lb=[0.]*len(wtVec),
                             ub=[0.]*len(wtVec), solver=self.solver,
                             solution=wtVec, weights=weights)
        else:
            ps = NonLinearProblem([[0.]], lb=[0.], ub=[0.], solver=self.solver)
            if weights is not None:
                if not isinstance(weights, ndarray):
                    weights=array(weights)
                ps.setObjective(lambda x : dot(x-wtVec, (x-wtVec)*weights))
            else:
                ps.setObjective(lambda x : dot(x-wtVec, x-wtVec))
        return ps.eval(solution)


def main():
    # 1. Parse command line

    usage = "Usage: %prog [options]"
    version = ("Minimization of metabolic adjustment\n" +
               COPYRIGHT_VERSION_STRING)
    parser = OptionParser(usage=usage, version=version)
    momaOptions = parser.add_option_group("General MOMA options")
    momaOptions.add_option("-r", "--reactions", dest="reactionFile",
                           help="perform MOMA on the mutant network given by "
                           "the reaction FILE", metavar="FILE")
    momaOptions.add_option("-p", "--parameters", dest="paramFile",
                           help="use the given scenario FILE for the mutant",
                           metavar="FILE")
    momaOptions.add_option("-w", "--wt-solution", dest="wtSolution",
                           help="use the given solution FILE for the wildtype",
                           metavar="FILE")
    momaOptions.add_option("-o", "--output", dest="outputFile",
                           help="write flux distribution computed by MOMA to "
                           "FILE", metavar="FILE")
    momaOptions.add_option("-v", "--reduce-by-fva", dest="redFvaFile",
                           help="use FVA result from FILE to reduce matrix "
                           "(experimental; only works if perturbed solution "
                           "space is a subset of the wildtype solution space)",
                           metavar="FILE")
    wmOptions = parser.add_option_group("Extra options for weighted MOMA")
    wmOptions.add_option("-x", "--wmoma", dest="wMomaFvaFile", help="perform "
                         "weighted MOMA with weights computed from the given "
                         "FVA solution FILE", metavar="FILE")
    wmOptions.add_option("-a", "--alpha", dest="alpha", type="float",
                         help="parameter ALPHA for computation of weights: "
                         "w = alpha + exp(-beta*(fvaMax-fvaMin)) (default: %g)"
                         % _DEFAULT_ALPHA)
    wmOptions.add_option("-b", "--beta", dest="beta", type="float",
                          help="parameter BETA for computation of weights: "
                          "w = alpha + exp(-beta*(fvaMax-fvaMin)) (default: %g)"
                          % _DEFAULT_BETA)
    optOptions = parser.add_option_group("Options for optimization")
    optOptions.add_option("-s", "--solver", dest="solver", help="QP/NLP solver "
                          "to be used for MOMA (default: cvxopt)")
    optOptions.add_option("-i", "--iterations", dest="numIter", type="int",
                          help="number of NLP runs to perform (from different "
                          "random start points; default: %u)" %
                          FbaParam.DEFAULT_NUMITER , metavar="N")
    optOptions.add_option("-l", "--use-full-matrix", action="store_true",
                          dest="useFullMatrix", help="use full matrix (disable "
                          "removal of dead ends, nonfunctional reactions, and "
                          "reactions with flux restricted to zero) - slow")
    parser.set_defaults(solver="default", useFullMatrix=False,
                        alpha=_DEFAULT_ALPHA, beta=_DEFAULT_BETA,
                        numIter=FbaParam.DEFAULT_NUMITER )

    options, _ = parser.parse_args()
    parser.check_required("-r")
    parser.check_required("-p")
    parser.check_required("-w")
    parser.check_required("-o")

    if options.wMomaFvaFile:
        if options.alpha < 0.:
            print "Error: alpha must be non-negative."
            exit()
        if options.beta <= 0.:
            print "Error: beta must be positive."
            exit()

    if options.numIter < 1:
        print "Error: Number of NLP runs must be positive."
        exit()

    # 2. Parse reaction file

    rparser = ReactionParser()
    model = MetabolicModel()
    try:
        model.addReactionsFromFile(options.reactionFile, rparser)
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(options.reactionFile))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("Error in reaction file %s:" %
               os.path.basename(options.reactionFile))
        print strerror
        exit()

    # 3. Parse scenario file

    model_messages = []
    pparser = ParamParser()
    try:
        lb, ub = pparser.parse(options.paramFile)[-2:]
        linConstraints = pparser.lin_constraints
        # Set flux bounds in model
        model.setFiniteBounds(lb, ub, True, model_messages)
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(options.paramFile))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("Error in scenario file %s:" %
               os.path.basename(options.paramFile))
        print strerror
        exit()
    except ValueError, strerror:
        print strerror
        exit()

    # 4. Parse solution file for wildtype

    wtFlux = MetabolicFlux()
    try:
        wtFlux.readFromFile(options.wtSolution)
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(options.wtSolution))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("An error occurred parsing file %s:" %
               os.path.basename(options.wtSolution))
        print strerror
        exit()

    if not wtFlux.hasSameReactions(model):
        print "Error: Solution and model must have the same reactions."
        exit()

    # 5. (Optionally) Read FVA solution for reducing the solution space
    #    - skip if full matrix is to be used

    if options.redFvaFile and not options.useFullMatrix:
        try:
            fbaSolution, redMinmax = \
                FvAnalyzer.parseSolutionFile(options.redFvaFile)
        except IOError, strerror:
            print ("An error occurred while trying to read file %s:" %
                   os.path.basename(options.redFvaFile))
            print strerror
            exit()
        except SyntaxError, strerror:
            print ("Error in FVA solution file %s:" %
                   os.path.basename(options.redFvaFile))
            print strerror
            exit()

        if not fbaSolution.hasSameReactions(model):
            print ("Error in file %s: FVA solution and model must have the same"
                   " reactions." % os.path.basename(options.redFvaFile))
            exit()

        # Determine blocked reactions based on FVA solution
        blockedReactions = FvAnalyzer.getBlockedReactions(redMinmax)

    elif options.useFullMatrix:
        blockedReactions = []
    else:
        # Determine blocked reactions based on dead-end analysis
        blockedReactions = model.findDeadEnds(True)[1]

    # 6. (Optionally) Read FVA solution and compute weights for wMOMA

    if options.wMomaFvaFile:
        try:
            fbaSolution, wmomaMinmax = \
                FvAnalyzer.parseSolutionFile(options.wMomaFvaFile)
        except IOError, strerror:
            print ("An error occurred while trying to read file %s:" %
                   os.path.basename(options.wMomaFvaFile))
            print strerror
            exit()
        except SyntaxError, strerror:
            print ("Error in FVA solution file %s:" %
                   os.path.basename(options.wMomaFvaFile))
            print strerror
            exit()

        if not fbaSolution.hasSameReactions(model):
            print ("Error in file %s: FVA solution and model must have the same"
                   " reactions." % os.path.basename(options.wMomaFvaFile))
            exit()

        weights = MomaAnalyzer.getWeightsFromFluxVar(model, wmomaMinmax,
                                                    options.alpha, options.beta)

    # 7. Perform MOMA against the wildtype solution on the metabolic network

    moma = MomaAnalyzer(options.solver)
    if options.wMomaFvaFile:
        print "Performing wMOMA with alpha = %g, beta = %g." % (options.alpha,
                                                                options.beta)
        distance, solution, status, ndim = moma.runOnModel(model, wtFlux,
            linConstraints, options.numIter, weights, blockedReactions)
    else:
        print "Performing MOMA..."
        distance, solution, status, ndim = moma.runOnModel(model, wtFlux,
            linConstraints, options.numIter, blockedReactions=blockedReactions)

    # Show warning and info messages of parsers
    msgs = (rparser.getMessages() + pparser.getMessages() +
            [x[1] for x in model_messages])
    if msgs:
        print '\n'+'\n'.join(msgs)
    print ("Info: The reduced network has %u reactions and %u metabolites." %
           ndim[1::-1])
    print "Solver status:", printStatus(status)

    # 8. Write output to file

    if len(solution) != 0:
        print "\nOptimal value of objective function:", distance
        diff = sum(solution.strictSqDiff(wtFlux).fluxDict.values())
        print "\nSquared distance to wildtype solution:", diff
        if model.biomass_name:
            print "Biomass:", solution[model.biomass_name]
        print "Total absolute flux:",
        print abs(array(solution.fluxDict.values())).sum()
        try:
            solution.writeToFile(options.outputFile)
        except IOError, strerror:
            print ("Unable to write to file %s:" %
                   os.path.basename(options.outputFile))
            print strerror
            exit()
    else:
        print "No output written."
        exit()


if __name__ == "__main__":
    main()



#!/usr/bin/env python

from metabolicmap import MetabolicMap
import re
import MySQLdb
import optparse
import sys

sys.path.append("/opt/metano/src")
try:
    from metabolicmodel import MetabolicModel
except ImportError:
    from metano.metabolicmodel import MetabolicModel

ecRegex = re.compile("-[0-9]+\.[0-9|-]+\.[0-9|-]+\.[0-9|-]+(_NADP?)?$")

class OptionParser(optparse.OptionParser):
    """ Extension of OptionParser with function for checking required arguments
    """

    def check_required(self, opt):
        option = self.get_option(opt)

        # Assumes the option's 'default' is set to None!
        if getattr(self.values, option.dest) is None:
            self.error("%s option not supplied" % option)

def main(argv=None):

#    server = "fileserver.bioinfo.nat.tu-bs.de"
#    user = "praktikant"
#    pw = "praktikant"
#    database = "metabolic_pathways"
    
    server = "134.169.105.249"
    user = "student"
    pw = "student"
    database = "sysbio"

    # command line options: for the selection of the model, 
    usage = "Usage: %prog [options]"
    parser = OptionParser(usage=usage)
    parser.add_option("-r","--reaction-file", dest="reactionFile",
                      help="File containing reactions in the metano format. EC numbers contained in the reaction names will be highlighted on the metabolic map", metavar="FILE")
    parser.add_option("-o","--output-file", dest="outputFile",
                      help="Name of the output file. Defaults to metabolicmap.gml", metavar="FILE", default="metabolicmap.gml")
    parser.add_option("-f","--font-size", dest="fontSize",
                      help="Font size of the labels in the metabolic map", metavar="FILE")
    parser.add_option("-v","--verbosity", dest="verbosityLevel", default=0,
                      help="Sets the verbosity level to 0 (no output), 1 (print status messages) or 2 (debug output)", metavar = "LEVEL")
    
    options, _ = parser.parse_args()
    parser.check_required("-r")
    
    if options.verbosityLevel:
        # TODO catch ValueError
        options.verbosityLevel=int(options.verbosityLevel)
    
    # read the metabolic model from file
    model = MetabolicModel()
    model.addReactionsFromFile(options.reactionFile)

    # parse EC numbers from reaction names
    ecList=[]
    for reaction in model:
        ecMatch = re.search(ecRegex, reaction.name)
        
        if ecMatch:
            ecNo = reaction.name[ecMatch.start():ecMatch.end()][1:]
            ecNo = re.sub("_NADP?","",ecNo)
            ecList.append(ecNo)

    # create the metabolic map
    theMap = MetabolicMap(server, user, pw, database, options.verbosityLevel)
    if options.fontSize:
        theMap.fontSize=options.fontSize
    
    theMap.drawEnzymes(ecList)
    theMap.drawRemainingReactions()
    theMap.drawPathwayLabels()
    theMap.writeToGML(options.outputFile)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

from metabolicmodel import MetabolicModel
import sys
import os

try:
    filename = sys.argv[1]
except IndexError:
    print "Error: No filename given."
    print "Usage is\n    " + os.path.basename(sys.argv[0]), "<reaction file>"
    exit()

model = MetabolicModel()
try:
    model.addReactionsFromFile(filename)
except IOError, strerror:
    print ("An error occurred while trying to read file %s:" %
           os.path.basename(filename))
    print strerror
    exit()
except SyntaxError, strerror:
    print ("Error in reaction file %s:" %
           os.path.basename(filename))
    print strerror
    exit()
print "\n".join(sorted(model.getMetaboliteNames()))



#!/usr/bin/env python

""" metano Script for checking transporters

This script checks all meaningful combinations of importers (e.g. C source,
N source, S source). It supports auxiliary importers of the same "type", e.g.
acetone import + glucose import as auxiliary transporter.


This file is part of metano.
Copyright (C) 2010-2014 Alexander Riemer,
Braunschweig University of Technology,
Dept. of Bioinformatics and Biochemistry

metano is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

metano is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with metano.  If not, see <http://www.gnu.org/licenses/>.
"""

from defines import FbaParam, COPYRIGHT_VERSION_STRING
from reactionparser import ReactionParser
from paramparser import ParamParser
from metabolicmodel import MetabolicModel
from transporterparser import TransporterParser
from fba import OptionParser, FbAnalyzer
from numpy import array, inf
import os

# debug flag: if True, the LB/UB values for every FBA are dumped to the console
debugFlag = False


def checkRecursive(resultList, currentList, otherLists, fba, matrix, reactions,
                   reaction_names, lb, ub, fbaParams, partialResult=[]):
    """ recursively enumerate all meaningful combinations of transporters and
        launch FBA on each such combination

    Keyword arguments:

    resultList     -- list for storage of results on innermost recursion level
    currentList    -- list of importers of the same type (perform FBA if empty)
    otherLists     -- list of lists of importers of other types (may be empty)
    fba            -- FbAnalyzer to be used for FBA
    matrix         -- the stoichiometric matrix of the metabolic network
    reactions      -- dictionary of all reactions { name : matrix column }
    reaction_names -- list of reaction names (column index -> name)
    lb             -- list of lower bounds (indexed like matrix columns)
    ub             -- list of upper bounds (indexed like matrix columns)
    fbaParams      -- ParamParser object for getting nonlinear constraints
    partialResult  -- growing result dictionary

    Returns: nothing, appends results to resultList

    resultList is list of dictionaries containing keys <type>, <type>_co,
    <type>_flux, <type>_co_flux for every importer and co-importer and key
    "solution" for the FBA solution
    """
    if currentList == []:
        # Recursion anchor: We have picked transporters of every type -> run FBA

        matrixSplit, reactionsSplit, lbSplit, ubSplit = \
            FbAnalyzer.splitFluxes(matrix, reaction_names, lb, ub)

        solutionSplit = fba.run(reactionsSplit, matrixSplit, lbSplit, ubSplit,
                                fbaParams)[1]

        keys = partialResult.keys()
        if solutionSplit == []:
            solution = []
            for key in keys:
                rea = partialResult[key]
                partialResult[key+"_flux"] = None
        else:
            solution = array(FbAnalyzer.rejoinFluxes(solutionSplit,
                                                     reactionsSplit,
                                                     reactions))
            for key in keys:
                rea = partialResult[key]
                if rea in reactions:
                    partialResult[key+"_flux"] = solution[reactions[rea]]
                else:
                    partialResult[key+"_flux"] = None

        partialResult["solution"] = solution

        # for debugging: store LB/UB as dictionary
        if debugFlag:
            lbub = {}
            for rea in reactions:
                rea_index = reactions[rea]
                lbub[rea] = (lb[rea_index], ub[rea_index])
            partialResult["bounds"] = lbub

        resultList.append(partialResult)
    else:
        # It's sufficient to query the first element as all members of
        # currentList[i] have the same type.
        typ = currentList[0][0]

        # Set next currentList for recursion (head of otherLists if not empty)
        if otherLists == []:
            nextList = []
        else:
            nextList = otherLists[0]

        for (_, trans_name, factor, cotrans) in currentList:
            # Make a copy of partialResult and extend by keys for current type
            tmpResult = dict(partialResult)
            tmpResult[typ] = trans_name
            tmpResult[typ+"_co"] = None

            if factor is not None:
                # Set flux through transporter to fix value
                index = reactions[trans_name]
                tmp_lb_trans = lb[index]
                tmp_ub_trans = ub[index]
                lb[index] = ub[index] = factor
                if cotrans is not None:
                    # make another copy of partial result
                    tmpResBak = dict(tmpResult)

                # First perform FBA without cotransporter
                checkRecursive(resultList, nextList, otherLists[1:], fba,
                               matrix, reactions, reaction_names, lb, ub,
                               fbaParams, tmpResult)

                # Now set flux through cotransporter to appropriate value
                # (factor)
                if cotrans is not None:
                    # get copy of partial result
                    tmpResult = tmpResBak
                    tmpResult[typ+"_co"] = cotrans

                    # Find factor cotrans_fac in currentList
                    for (_, cotrans_name, cotrans_fac, _) in currentList:
                        if cotrans_name == cotrans:
                            break

                    co_index = reactions[cotrans]
                    tmp_lb_cotrans = lb[co_index]
                    tmp_ub_cotrans = ub[co_index]
                    lb[co_index] = ub[co_index] = cotrans_fac

                    checkRecursive(resultList, nextList, otherLists[1:], fba,
                                   matrix, reactions, reaction_names, lb, ub,
                                   fbaParams, tmpResult)

                    lb[co_index] = tmp_lb_cotrans
                    ub[co_index] = tmp_ub_cotrans

                lb[index] = tmp_lb_trans
                ub[index] = tmp_ub_trans
            else:
                # Allow arbitrary flux through transporter
                index = reactions[trans_name]
                tmp_lb_trans = lb[index]
                tmp_ub_trans = ub[index]
                lb[index] = -inf
                ub[index] =  inf

                # First perform FBA without cotransporter
                checkRecursive(resultList, nextList, otherLists[1:], fba,
                               matrix, reactions, reaction_names, lb, ub,
                               fbaParams, tmpResult)

                # Now also allow arbitrary flux through cotransporter
                if cotrans is not None:
                    # make another copy of partial result
                    tmpResult = dict(tmpResult)
                    tmpResult[typ+"_co"] = cotrans
                    co_index = reactions[cotrans]
                    tmp_lb_cotrans = lb[co_index]
                    tmp_ub_cotrans = ub[co_index]
                    lb[co_index] = -inf
                    ub[co_index] =  inf

                    checkRecursive(resultList, nextList, otherLists[1:], fba,
                                   matrix, reactions, reaction_names, lb, ub,
                                   fbaParams, tmpResult)

                    lb[co_index] = tmp_lb_cotrans
                    ub[co_index] = tmp_ub_cotrans

                lb[index] = tmp_lb_trans
                ub[index] = tmp_ub_trans


def main():
    # 1. Parse command line

    usage = "Usage: %prog [options]"
    version = "Transporter check\n" + COPYRIGHT_VERSION_STRING
    parser = OptionParser(usage=usage, version=version)
    parser.add_option("-r", "--reactions", dest="reactionFile",
                      help="perform Flux Balance Analysis on the network given "
                           "by the reaction FILE", metavar="FILE")
    parser.add_option("-p", "--parameters", dest="paramFile",
                      help="use the given scenario FILE for Flux Balance "
                           "Analysis", metavar="FILE")
    parser.add_option("-t", "--transporters", dest="transFile",
                      help="use list of transporters from FILE", metavar="FILE")
    parser.add_option("-o", "--output", dest="outputFile",
                      help="write output of Flux Balance Analysis to FILE",
                      metavar="FILE")

    options, _ = parser.parse_args()
    parser.check_required("-r")
    parser.check_required("-p")
    parser.check_required("-t")
    parser.check_required("-o")

    # 2. Parse reaction file

    rparser = ReactionParser()
    model = MetabolicModel()
    try:
        model.addReactionsFromFile(options.reactionFile, rparser)
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(options.reactionFile))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("Error in reaction file %s:" %
               os.path.basename(options.reactionFile))
        print strerror
        exit()
    reactions = model.reactionDict
    matrix = array(model.getStoichiometricMatrix())

    # 3. Parse scenario file

    model_messages = []
    pparser = ParamParser()
    try:
        # Parse file, get maxmin, name of objective function, and solver name
        maxmin, obj_name, solver, numIter, lb , ub = pparser.parse(
                                                        options.paramFile)
        fbaParams = FbaParam(solver, maxmin, obj_name, numIter)
        # Set flux bounds in model
        model.setFiniteBounds(lb, ub, True, model_messages)
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(options.paramFile))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("Error in scenario file %s:" %
               os.path.basename(options.paramFile))
        print strerror
        exit()
    except ValueError, strerror:
        print strerror
        exit()
    lb, ub = map(array, model.getBounds())

    # Show warning and info messages of parsers
    msgs = (rparser.getMessages() + pparser.getMessages() +
            [x[1] for x in model_messages])
    if msgs:
        print '\n'+'\n'.join(msgs)

    # 4. Parse transporter file

    tparser = TransporterParser()
    try:
        transporters = tparser.parse(options.transFile)
        transpByType = tparser.split_by_type()
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(options.transFile))
        print strerror
        exit()
    except SyntaxError, strerror:
        print strerror
        exit()
    except ValueError, strerror:
        print strerror
        exit()

    # 5. Set all transporter fluxes to zero

    for (trans_name, _, _) in transporters:
        if trans_name not in reactions:
            print ("Error: Transporter %s is not a reaction of the network." %
                   trans_name)
            exit()
        index = reactions[trans_name]
        lb[index] = ub[index] = 0.

    # 6. Identify main ("carbon") source - the one that has factors

    main_type = None
    for entry in transpByType:
        typ = entry[0][0]
        index = 0
        for (_, _, factor, _) in entry:

            if main_type is None and factor is not None:
                if index != 0:
                    print ("Error: Main transporter type must have flux "
                           "factors. (Identified main type '%s')" % main_type)
                    exit()
                main_type = typ

            elif typ == main_type and factor is None:
                print ("Error: Main transporter type must have flux "
                       "factors. (Identified main type '%s')" % main_type)
                exit()

            elif typ != main_type and factor is not None:
                print ("Error: Only one transporter type can have factors.\n"
                       "Found factors with types %s and %s" % (main_type, typ))
                exit()

            index += 1

    for i in range(len(transpByType)):
        if transpByType[i][0][0] == main_type:
            main_index = i
            break

    # 7. Perform flux balance analysis for all combinations of transporters

    if solver == "":
        solver = "default"
    fba = FbAnalyzer(solver)

    resultList = []
    checkRecursive(resultList, transpByType[main_index],
        transpByType[:main_index]+transpByType[main_index+1:], fba, matrix,
        reactions, model.getReactionNames(), lb, ub, fbaParams)

    # For debugging: Output lb/ub to console
    if debugFlag:
        keys = resultList[0].keys()
        keys.remove("solution")
        keys.remove("bounds")
        keys.sort()
        for result in resultList:
            print "***",
            for key in keys:
                print key+":", result[key],
            print
            lbub = result["bounds"]
            for rea in sorted(lbub):
                print rea, lbub[rea]

    # 8. Write output to file

    try:
        write_output(options.outputFile, reactions, resultList, main_type)
    except IOError, strerror:
        print ("Unable to write to file %s:" %
               os.path.basename(options.outputFile))
        print strerror
        exit()


def write_output(filename, reactions, resultList, main_type):
    if resultList == []:
        return  # Nothing to write

    # Get all types
    types = []
    for key in resultList[0]:
        try:
            if not key.endswith("_co") and resultList[0][key] in reactions:
                types.append(key)
        except TypeError:
            # Skip non-hashable values of resultList[0][key], such as dicts or
            # lists
            pass

    # Get length of longest reaction name (for formatting)
    maxtypelen = len(max(types, key=len))+len("_cosource")
    maxlen = max(len(max(reactions, key=len)), maxtypelen)

    # Determine optimal column widths
    col_width = []
    for i in range(len(resultList)):
        width = 12
        for typ in types:
            width = max(width, len(resultList[i][typ]))
            if resultList[i][typ+"_co"] in reactions:
                width = max(width, len(resultList[i][main_type+"_co"]))
        col_width.append(width)

    with open(filename, 'w') as f:
        # First line: transporter - main_type first
        line = (main_type+"_source").ljust(maxlen)+" :"
        for i in range(len(resultList)):
            line += ' ' + resultList[i][main_type].rjust(col_width[i])
        f.write(line+'\n')

        # Second line: cotransporter
        line = (main_type+"_cosource").ljust(maxlen)+" :"
        any_cotrans = False
        for i in range(len(resultList)):
            if resultList[i][main_type+"_co"] in reactions:
                s = resultList[i][main_type+"_co"]
                any_cotrans = True
            else:
                s = "N/A"
            line += ' ' + s.rjust(col_width[i])
        if any_cotrans:
            f.write(line+'\n')

        # Third line: main type transporter flux
        key = main_type+"_flux"
        line = key.ljust(maxlen)+" :"
        for i in range(len(resultList)):
            val = resultList[i][key]
            if val == None:
                val = 0.
            line += ' ' + ("% .6g" % val).rjust(col_width[i])
        f.write(line+'\n')

        # Fourth line: main type cotransporter flux
        key = main_type+"_co_flux"
        line = key.ljust(maxlen)+" :"
        any_cotrans = False
        for i in range(len(resultList)):
            val = resultList[i][key]
            if val == None:
                s='N/A'
            else:
                s = "% .6g" % val
                any_cotrans = True
            line += ' ' + s.rjust(col_width[i])
        if any_cotrans:
            f.write(line+'\n')

        # Repeat for all other types
        types.remove(main_type)

        for typ in types:
            # Transporter
            line = (typ+"_source").ljust(maxlen)+" :"
            for i in range(len(resultList)):
                line += ' ' + resultList[i][typ].rjust(col_width[i])
            f.write(line+'\n')

            # Cotransporter
            line = (typ+"_cosource").ljust(maxlen)+" :"
            any_cotrans = False
            for i in range(len(resultList)):
                if resultList[i][typ+"_co"] in reactions:
                    s = resultList[i][typ+"_co"]
                    any_cotrans = True
                else:
                    s = "N/A"
                line += ' ' + s.rjust(col_width[i])
            if any_cotrans:
                f.write(line+'\n')

            # Transporter flux
            key = typ+"_flux"
            line = key.ljust(maxlen)+" :"
            for i in range(len(resultList)):
                val = resultList[i][key]
                if val == None:
                    val = 0.
                line += ' ' + ("% .6g" % val).rjust(col_width[i])
            f.write(line+'\n')

            # Cotransporter flux
            key = typ+"_co_flux"
            line = key.ljust(maxlen)+" :"
            any_cotrans = False
            for i in range(len(resultList)):
                val = resultList[i][key]
                if val == None:
                    s='N/A'
                else:
                    s = "% .6g" % val
                    any_cotrans = True
                line += ' ' + s.rjust(col_width[i])
            if any_cotrans:
                f.write(line+'\n')

        # Flux distributions ('N/A' in all rows if solution is empty)
        f.write('\n')
        for rea in sorted(reactions):
            index = reactions[rea]
            line = rea.ljust(maxlen)+" :"
            for i in range(len(resultList)):
                try:
                    s = "% .6g" % resultList[i]["solution"][index]
                except IndexError:
                    s = "N/A"
                line += ' ' + s.rjust(col_width[i])
            f.write(line+'\n')


if __name__ == "__main__":
    main()



"""
This module defines a number of global constants and helper functions.


This file is part of metano.
Copyright (C) 2010-2014 Alexander Riemer,
Braunschweig University of Technology,
Dept. of Bioinformatics and Biochemistry

metano is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

metano is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with metano.  If not, see <http://www.gnu.org/licenses/>.
"""

METANO_VERSION = "1.0.1.2"

COPYRIGHT_VERSION_STRING = ("metano v" + METANO_VERSION + "\n"
                            "Copyright (C) 2010-2014 Alexander Riemer,\n"
                            "Braunschweig University of Technology,\n"
                            "Dept. of Bioinformatics & Biochemistry.\n\n"
                            "This program comes with ABSOLUTELY NO WARRANTY.\n"
                            "metano is free software distributed under the GNU "
                            "General Public License v3,\nand you are welcome to"
                            " redistribute it under the conditions of that "
                            "license.")

from numpy import isinf

class Verbosity:
    """ Levels of verbosity
    """
    NONE           = 0
    ERROR          = 1
    WARNING        = 2
    INFO           = 3
    DEBUG          = 4
    ALL            = 4

class When:
    """ Constants for when to perform an action
    """
    NEVER          = 0
    ONCE           = 1
    AUTO           = 2
    ALWAYS         = 3
    ANY            = set((NEVER, ONCE, AUTO, ALWAYS))

def decodeWhen(whenStr, allowedOptions=When.ANY):
    """ decode string specifying when to perform an option
    """
    when = { "never"  : When.NEVER,
             "once"   : When.ONCE,
             "auto"   : When.AUTO,
             "always" : When.ALWAYS }[whenStr.lower()]

    if when not in allowedOptions:
        raise KeyError(whenStr)
    return when


class SolverStatus:
    """ Status of optimization
    """
    PREPARED       = -1
    OPTIMAL        =  0
    PRIM_INFEAS    =  1
    DUAL_INFEAS    =  2
    UNBOUNDED      =  3
    TIMEOUT        =  4
    MAXITER        =  5
    NOCONV         =  6
    ERROR          =  7
    UNKNOWN        =  8

def printStatus(status):
    try:
        return { SolverStatus.PREPARED : 'not solved, yet',
                 SolverStatus.OPTIMAL : 'optimal',
                 SolverStatus.PRIM_INFEAS : 'primal infeasible',
                 SolverStatus.DUAL_INFEAS : 'dual infeasible',
                 SolverStatus.UNBOUNDED : 'unbounded',
                 SolverStatus.TIMEOUT : 'time limit exhausted',
                 SolverStatus.MAXITER : 'iteration limit exhausted',
                 SolverStatus.NOCONV : 'not converged',
                 SolverStatus.ERROR : 'error',
                 SolverStatus.UNKNOWN : 'unknown (not converged)' }[status]
    except KeyError:
        return "unexpected value: %r" % status


def cvxpyToSolverStatus(status):
    if isinf(status):
        return SolverStatus.PRIM_INFEAS
    else:
        return SolverStatus.OPTIMAL


def cvxToSolverStatus(status):
    try:
        return { 'optimal' : SolverStatus.OPTIMAL,
                 'primal infeasible' : SolverStatus.PRIM_INFEAS,
                 'dual infeasible' : SolverStatus.DUAL_INFEAS,
                 'unknown' : SolverStatus.UNKNOWN }[status]
    except KeyError:
        return SolverStatus.UNKNOWN


def glpkToSolverStatus(status):
    try:
        return { 'solution is optimal' : SolverStatus.OPTIMAL,
                 'solution is feasible' : SolverStatus.OPTIMAL,
                 'solution is infeasible' : SolverStatus.PRIM_INFEAS,
                 'problem has no feasible solution' : SolverStatus.PRIM_INFEAS,
                 'problem has unbounded solution' : SolverStatus.UNBOUNDED,
                 'solution is undefined' : SolverStatus.UNKNOWN,
                 'e_fault' : SolverStatus.ERROR,
                 'e_objll' : SolverStatus.UNBOUNDED,
                 'e_objul' : SolverStatus.UNBOUNDED,
                 'e_itlim' : SolverStatus.MAXITER,
                 'e_tmlim' : SolverStatus.TIMEOUT,
                 'e_nofeas' : SolverStatus.PRIM_INFEAS,
                 'e_instab' : SolverStatus.ERROR,
                 'e_sing' : SolverStatus.ERROR,
                 'e_noconv' : SolverStatus.NOCONV,
                 'e_nopfs' : SolverStatus.PRIM_INFEAS,
                 'e_nodfs' : SolverStatus.DUAL_INFEAS }[status]
    except KeyError:
        return SolverStatus.UNKNOWN


class ReaFileStruc:
    """ Definitions for structure (syntax & some semantics) of reaction file
    """
    delimiter   = ":"    # delimiter separating reaction name and equation
    arrowIrr    = "-->"  # arrow for irreversible reactions
    arrowFlip   = "<--"  # arrow for right-to-left irreversible reactions
    arrowRev    = "<=>"  # arrow for reversible reactions
    commentSign = "#"    # comment sign (text right of sign is ignored)

    class ArrowProperties(object):
        """ Class for storing the properties of a reaction arrow

            Members:
                length     -- length of the arrow string in characters
                reversible -- reversibility flag (False = irreversible reaction)
                flipped    -- flipped flag (True if an irreversible reaction is
                              flipped, i.e. given in reversed order (products on
                              left-hand side, reactants on right-hand side)
        """
        def __init__(self, length=0, reversible=False, flipped=False):
            self.length = length
            self.reversible = reversible
            self.flipped = flipped

        def __str__(self):
            s = "(%u, " % self.length
            if not self.reversible:
                s += "ir"
            s += "reversible, "
            if self.flipped:
                s += "flipped)"
            else:
                s += "normal)"
            return s

    arrow_prop = {arrowIrr : ArrowProperties(len(arrowIrr), False, False),
                  arrowFlip: ArrowProperties(len(arrowFlip), False, True),
                  arrowRev : ArrowProperties(len(arrowRev), True,  False)}


class FbaParam(object):
    """ data structure for passing FBA parameter sets

    Member variables:

    solver           -- name of the solver to be used for the optimization
    maxmin           -- maximize (True) or minimize (False) objective function
    objStr           -- objective function definition (given as string)
    linConstraints   -- list of LinearConstraint objects
    numIter          -- number of iterations in case of nonlinear optimization
    nlc              -- list of nonlinear constraints (lambda functions)
    nlc_grad         -- gradients of nonlinear constraints
    """
    DEFAULT_NUMITER = 10

    def hasEqualityConstraints(self):
        for c in self.linConstraints:
            if c.isEq:
                return True
        return False

    def __init__(self, solver="default", maxmin=True, objStr="", numIter=-1):
        self.solver  = solver
        self.maxmin  = maxmin
        self.objStr  = objStr
        self.numIter = -1
        self.linConstraints = []
        self.nlc = []
        self.nlc_grad = []

    def setLinConstraints(self, lc):
        self.linConstraints = lc

    def setNLConstraints(self, nlc, nlc_grad):
        self.nlc = nlc
        self.nlc_grad = nlc_grad

    def __repr__(self):
        return "FbaParam(%r, %r, %r, %r)" % (self.solver, self.maxmin,
                                             self.objStr, self.numIter)


def typename(x):
    """ return the most informative type name available for x
    """
    typeOfX = type(x).__name__
    if typeOfX == "instance":
        return x.__class__.__name__
    return typeOfX


def printHistogram(histogram, minval, maxval, revert=False):
    numBins = len(histogram)
    if not numBins:
        return
    if revert:
        tmp = -minval
        minval = -maxval
        maxval = tmp

    print ("\nmin value (0): %.12g, max value (%u): %.12g\nHistogram:\n" %
           (minval, numBins-1, maxval))

    binWidth = (maxval-minval)/float(numBins)
    if revert:
        # Print histogram negated and in reverse order
        for i in range(numBins):
            print ("%.12g" % (minval + float(i)*binWidth)).ljust(16),
            print -histogram[-i-1]
    else:
        for i in range(numBins):
            print ("%.12g" % (minval + float(i)*binWidth)).ljust(16),
            print histogram[i]
    print ("%.12g" % maxval).ljust(16)


def exportMatrixAsDotM(matrix, filename):
    """ export a matrix as .m file (for Matlab/Octave)
    """
    with open(filename, 'w') as f:
        exportMatrixAsDotMByHandle(matrix, f)


def exportMatrixAsDotMByHandle(matrix, f):
    """ export a matrix as .m file (for Matlab/Octave) to the given file handle
        (must be open for writing)
    """
    nRows = len(matrix)
    if not nRows:
        return      # nothing to do

    f.write("S = [")
    i = 0
    for row in matrix:
        f.write(' '.join(map(repr, row)))
        i += 1
        if i >= nRows:
            f.write(']')
        f.write(";\n")


def makeCoefNameTerm(s, coef, name):
    """ generate a string representation of the (coef, name) pair with an
        appropriate sign connecting the term to preceding terms

    Keyword arguments:

    s    -- string containing the preceding terms (may be empty)
    coef -- coefficient (positive or negative floating-point value)
    name -- name string to be displayed next to coefficient
    """
    # Suppress terms with a coefficient of zero, and omit coefficient 1.
    if coef == 0.:
        return ""
    absCoef = abs(coef)
    if absCoef == 1.:
        absCoefTerm = "%s" % name
    else:
        absCoefTerm = "%r %s" % (absCoef, name)

    # First coefficient is marked with minus sign or nothing, others with " + "
    # or " - "
    if s == "":
        if coef < 0:
            sign = "-"
        else:
            sign = ""
    else:
        if coef < 0:
            sign = " - "
        else:
            sign = " + "

    return sign + absCoefTerm


def displayPermutedMatrix(matrix, permutation, names):
    """ return a list of lines for displaying the given sparsely populated
        matrix

    Output is in form of linear combinations with the coefficients coming from
    the matrix and the variables coming from the list of names. Example:

      displayPermutedMatrix([[0, 1, -1]], [1, 0, 2], ["A", "B", "C"]) yields
      ["1 A - 1 C"]

    Keyword arguments:

    matrix      -- matrix
    permutation -- permutation[i] is the entry in names for matrix column i
    names       -- list of reaction names
    """
    nCols = len(permutation)
    lines = []
    for row in matrix:
        s = ""
        for i in range(nCols):
            coef = row[i]
            if coef:
                s += makeCoefNameTerm(s, coef, names[permutation[i]])
        lines.append(s)
    return lines


def makeUnique(strings):
    """ make a list of strings unique by appending arbitrary numbers,
        e.g. if string "abc" appears twice in the list, the occurrences are
        replaced with "abc_001" and "abc_002"
    """
    result = list(strings)  # copy list
    nStrings = len(strings)
    sDict = {}  # dict { string : list of indices }
    for i in range(nStrings):
        name = strings[i]
        if name in sDict:
            sDict[name].append(i)
        else:
            sDict[name] = [i]

    if len(sDict) != nStrings:
        for name in sDict:
            indexList = sDict[name]
            if len(indexList) > 1:
                for i in range(len(indexList)):
                    # simply add number to name
                    result[indexList[i]] += "_%03u" % (i+1)
    return result


def addToDictPlus(d, key, value):
    """ if key doesn't exist, add (key : value) pair to d, otherwise add value
        to d[key]
    """
    if key in d:
        d[key] += value
    else:
        d[key] = value


def roundAndCut(number, ndigits=8, threshold=1E-8):
    """ round a number to ndigits digits and set it to zero if its absolute
        value is below threshold
    """
    if abs(number) < threshold:
        return 0.
    else:
        return round(number, ndigits)

def isZeroVector(vec):
    for elem in vec:
        if elem != 0.:
            return False
    return True

def padNumber(numStr):
    """ pad a number given as a string representation by adding a space to the
        left if the number is not negative - string must not be empty
    """
    if numStr[0] == '-':
        return numStr
    else:
        return ' ' + numStr



"""
Convex optimization modeling for cvxopt.

"""

# Copyright (C) 2006-2008 Jacob Mattingley and Stephen Boyd.
#
# This file is part of CVXMOD.
#
# CVXMOD is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
#
# CVXMOD is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.

import cvxmod
from cvxmod.base import *
from cvxmod.base import lhsrhs
from cvxmod.errors import *
# Next imports are used when testing values of inequalities.
from cvxmod.base import eps, _maxeignum, _mineignum

__all__ = ["cvxmodset", "lt", "gt", "In"]
LT = '|lt|'
GT = '|gt|'

class setrelation(object):
    # jem. lousy name?
    def __init__(self, expr, js):
        if not equiv(size(expr), size(js)):
            raise DimError
        self.expr = expr
        self.js = js

    def __repr__(self):
        # jem. mention the size? set relation in R^{x,y}?
        return '<set relation %s |In| %s; optvars: %s>' % \
                (str(self.expr), str(self.js),
                 stroptvars(getoptvars(self)))

    def split(self):
        return self.js.constrform(self.expr)

    # jem. bad method for determining convexity?
    def _getconvex(self):
        # jem: is this the whole deal?
        return isaffine(self.expr)
    convex = property(_getconvex)

    def getoptvars(self):
        return getoptvars(self.expr)

    def getassertions(self):
        return getassertions(self.expr)

    def _getvalue(self):
        return value(self.split())
    value = property(_getvalue)

    def __nonzero__(self):
        raise OptimizationError("wrap value() around set relations if comparing values")

class exprIncombination(object):
    def __init__(self, expr):
        self.expr = expr

    def __or__(self, other):
        if isinstance(other, cvxmodset):
            return setrelation(self.expr, other)
        else:
            raise TypeError('invalid type for half setrelation')

    def __repr__(self):
        return '<half of a setrelation>'

class Incvxmodsetcombination(object):
    def __init__(self, js):
        self.js = js

    def __or__(self, other):
        if isinstance(other, (function, symbol)):
            return setrelation(other, self.js)

    def __repr__(self):
        return '<half of a setrelation>'

class cvxmodset(object):
    pass

class Intype(object):
    def __or__(self, other):
        if isinstance(other, (function, symbol)):
            return exprIncombination(other)
        elif isinstance(other, cvxmodset):
            return Incvxmodsetcombination(other)
        else:
            raise TypeError('invalid type for |In| pseudo-operator')

    def __ror__(self, other):
        if isinstance(other, cvxmodset):
            return Incvxmodsetcombination(other)
        else:
            return exprIncombination(other)

    def __repr__(self):
        return '<pseudo-operator for set membership>'

In = Intype()
class genineq(lhsrhs):
    def __init__(self, lhs, rhs, sign):
        if is1x1(lhs) or is1x1(rhs) or equiv(size(lhs), size(rhs)):
            self.lhs = lhs
            self.rhs = rhs
            self.sign = sign
            if is1x1(lhs):
                self.rows = rows(rhs)
                self.cols = cols(rhs)
            else:
                self.rows = rows(lhs)
                self.cols = cols(lhs)
        else:
            raise DimError

    def __repr__(self):
        return "<%sx%s generalized inequality %s %s %s; optvars: %s>" % \
            (str(rows(self)), str(cols(self)), str(self.lhs), self.sign,
            str(self.rhs), stroptvars(self))

    def __str__(self):
        return "%s %s %s" % (str(self.lhs), self.sign, str(self.rhs))

    def stdlhs(self):
        if self.sign is LT:
            return self.lhs - self.rhs
        elif self.sign is GT:
            return self.rhs - self.lhs
        else:
            raise NotImplementedError("cannot find stdlhs for this genineq")

    def _getconvex(self):
        return isaffine(self.stdlhs())
    convex = property(_getconvex)

    def split(self):
        stdlhs = self.stdlhs()
        if not issymm(stdlhs):
            xs = dummyvar(size(stdlhs), symm=True)
            s = (xs == stdlhs).split()
            s += (xs |lt| 0).split()
            return s
        else:
            return stdstruct(psdcone=[stdlhs |lt| 0])

    def _getvalue(self):
        return _maxeignum(value(self.stdlhs())) <= eps
    value = property(_getvalue)

    def cvx(self):
        return "%s == semidefinite(%s)" % (str(-self.stdlhs()),
                                           str(rows(self.stdlhs())))

class genineqlhs(object):
    def __init__(self, lhs, sign):
        self.lhs = lhs
        self.sign = sign

    def __or__(self, other):
        # jem: need to test if we have a cvxmod object.
        return genineq(self.lhs, other, self.sign)

    def __repr__(self):
        return '<left-hand side of a generalized inequality>'

class genineqrhs(object):
    def __init__(self, rhs, sign):
        self.rhs = rhs
        self.sign = sign

    def __or__(self, other):
        if isinstance(other, (function, symbol)):
            return genineq(other, self.rhs, self.sign)

    def __repr__(self):
        return '<right-hand side of a generalized inequality>'

class ltrel(object):
    pass

class lttype(object):
    def __or__(self, other):
        if isaffine(other):
            return genineqrhs(other, LT)
        else:
            raise TypeError('invalid type for |lt|: must be affine')

    def __ror__(self, other):
        if isaffine(other):
            return genineqlhs(other, LT)
        else:
            raise TypeError('invalid type for |lt|: must be affine')

    def __repr__(self):
        return '<pseudo-operator for PSD cones>'
lt = lttype()
class gttype(object):
    def __or__(self, other):
        if isaffine(other):
            return genineqrhs(other, GT)
        else:
            raise TypeError('invalid type for |gt|: must be affine')

    def __ror__(self, other):
        if isaffine(other):
            return genineqlhs(other, GT)
        else:
            raise TypeError('invalid type for |gt|: must be affine')

    def __repr__(self):
        return '<pseudo-operator for PSD cones>'
gt = gttype()



#!/usr/bin/env python

""" Script for adding bounds imposed by the irreversibility of some reactions to
    a metano scenario file

    This script reads a scenario file and a reaction file and writes a scenario
    file. While the two scenario files can be the same, it is advisable to keep
    the original scenario file, which is otherwise overwritten.

    The central function is performed by
    MetabolicModel.writeBoundsToParamFileHandle().
"""

from fba import OptionParser
from reactionparser import ReactionParser
from paramparser import ParamParser
from metabolicmodel import MetabolicModel
from defines import COPYRIGHT_VERSION_STRING
import os


def main():
    # 1. Parse command line

    usage = "Usage: %prog [options]"
    version = "%prog\n" + COPYRIGHT_VERSION_STRING
    parser = OptionParser(usage=usage, version=version)
    parser.add_option("-p", "--parameters", dest="inputFile",
                      help="use the given scenario FILE", metavar="FILE")
    parser.add_option("-r", "--reactions", dest="reactionFile", help="use "
                      "reversibilities from reaction FILE", metavar="FILE")
    parser.add_option("-o", "--output", dest="outputFile", help="write output "
                      "(scenario file) to FILE", metavar="FILE")

    options, _ = parser.parse_args()
    parser.check_required("-p")
    parser.check_required("-r")
    parser.check_required("-o")

    # 2. Parse reaction file

    rparser = ReactionParser()
    model = MetabolicModel()
    try:
        model.addReactionsFromFile(options.reactionFile, rparser)
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(options.reactionFile))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("Error in reaction file %s:" %
               os.path.basename(options.reactionFile))
        print strerror
        exit()

    # 3. Parse scenario file

    model_messages = []
    pparser = ParamParser()
    try:
        # Parse file, get maxmin, name of objective function, and solver name
        maxmin, obj_name, solver, numIter, lb , ub = \
            pparser.parse(options.inputFile)
        # Set flux bounds in model (for parameter check)
        model.setFiniteBounds(lb, ub, True, model_messages)
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(options.inputFile))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("Error in scenario file %s:" %
               os.path.basename(options.inputFile))
        print strerror
        exit()
    except ValueError, strerror:
        print strerror
        exit()

    # Show warning and info messages of parsers
    msgs = (rparser.getMessages() + pparser.getMessages() +
            [x[1] for x in model_messages])
    if msgs:
        print '\n'.join(msgs)

    # 4. Write new scenario file

    try:
        with open(options.outputFile, 'w') as f:
            str_maxmin = {True : "max", False : "min"}[maxmin]
            f.write("OBJ %s %s\n" % (str_maxmin, obj_name))
            if solver != "":
                f.write("SOLVER %s\n" % solver)
            if numIter >= 0:
                f.write("NUM_ITER %u\n" % numIter)
            f.write("\n")
            model.writeBoundsToParamFileHandle(f)
            f.write("\n")
    except IOError, strerror:
        print ("An error occurred while trying to write file %s:" %
               os.path.basename(options.outputFile))
        print strerror
        exit()


if __name__ == "__main__":
    main()



"""
General utility functions that don't belong in cvxmod.base.

"""

# Copyright (C) 2006-2008 Jacob Mattingley and Stephen Boyd.
#
# Portions of this file were contributed by Timothy Hunter.
#
# This file is part of CVXMOD.
#
# CVXMOD is free software; you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation; either version 3 of the License, or (at your option) any later
# version.
#
# CVXMOD is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.

import cvxmod
from cvxmod.base import *

import cvxopt
from cvxopt import lapack
from cvxopt import cholmod

from math import sqrt

__all__ = ["randn", "rand", "randseed", "linspace", "joinlist", "iterable",
           "bylowerstr", "slicetoindices", "sqrtm", "svd", "eig",
           "leastsquares", "leastnorm", "cholsolve", "spy", "cholnum"]

def randn(rows=1, cols=None, mean=0, std=1):
    if iterable(rows):
        if len(rows) == 2:
            cols = rows[1]
            rows = rows[0]
        else:
            raise TypeError('invalid dimension tuple')

    if cols is None:
        cols = rows

    if equiv((rows, cols), (1, 1)):
        return float(cvxopt.base.normal(1, 1, mean=mean, std=std)[0])
    else:
        return cvxopt.base.normal(rows, cols, mean, std)

def rand(rows=1, cols=None, lowerlim=0, upperlim=1):
    if iterable(rows):
        if len(rows) == 2:
            cols = rows[1]
            rows = rows[0]
        else:
            raise TypeError('invalid dimension tuple')

    if cols is None:
        cols = rows

    if equiv((rows, cols), (1, 1)):
        return float(cvxopt.base.uniform(1, 1, lowerlim, upperlim)[0])
    else:
        return cvxopt.base.uniform(rows, cols, lowerlim, upperlim)

def linspace(start, end, N, incstop=True):
    x = zeros(N, 1, full=True)
    if incstop:
        step = (end - start) / (N - 1)
    else:
        step = (end - start) / N

    x[0] = start
    for i in range(1, N):
        x[i] = x[i-1] + step

    return x

def randseed(s=None):
    if s is None:
        cvxopt.base.setseed()
    else:
        cvxopt.base.setseed(s)

def joinlist(l, skipand=False):
    if not l:
        return ''
    elif len(l) == 1:
        return str(l[0])
    elif skipand:
        return ', '.join([str(x) for x in l])
    else:
        s = ''
        for x in l[:-2]:
            s += str(x) + ', '
        s += str(l[-2]) + ' and ' + str(l[-1])
        return s

def iterable(obj):
    """Returns True if obj is iterable, i.e., if obj is a sequence."""
    if isinstance(obj, (matrix, spmatrix)):
        return False
    try:
        len(obj)
        return True
    except TypeError:
        return False

def bylowerstr(l):
    return sorted(l, lambda x, y: cmp(str(x).lower(), str(y).lower()))

def slicetoindices(s, l):
    # takes a slice, a list or a constant, and returns a list of indices.
    # l is the length of the relevant object into which we are indexing.
    l = value(l)
    if isinstance(s, slice):
        # syntax is {range|slice}(start, stop, step).
        start = value(s.start)
        stop = value(s.stop)
        step = value(s.step)

        if start is None:
            start = 0

        if stop is None:
            stop = l

        if step is None:
            step = 1

        if stop < 0:
            stop = l + stop

        return range(start, stop, step)

    elif iterable(s):
        return [value(x) for x in s]
    
    else:
        return [value(s)]

def sqrtm(A):
    """Returns the matrix square root of a positive semidefinite matrix."""
    if not isinstance(A, (matrix, spmatrix)) or rows(A) != cols(A) or eig(A)[0][0] < 0:
        raise TypeError('a symmetric positive semidefinite matrix is required')

    V = matrix(A)
    z = zeros(rows(A), 1, full=True)
    lapack.syev(V, z, jobz='V')

    # Round eigenvalues to deal with numerical instability.
    # Note: don't use cvxmod atoms pos or sqrt here: overkill.
    for i in range(len(z)):
        if z[i] <= 0:
            z[i] = 0
        else:
            z[i] = sqrt(z[i])

    return V*diag(z)*tp(V)

def leastsquares(A, B):
    """Finds the least-squares approximate solution X, which minimizes ||AX -
    B||_fro.
    
    A must be full rank."""

    if not (isinstance(A, (matrix, spmatrix)) and isinstance(B, (matrix, spmatrix))):
        raise TypeError('both arguments must be matrices')

    if rows(A) < cols(A):
        raise DimError('A must be skinny')

    if rows(B) != rows(A):
        raise DimError('rows(B) must equal rows(A)')

    X = matrix(B)
    lapack.gels(matrix(A), X)
    return X

def leastnorm(A, B):
    """Finds the least-norm X which minimizes ||X||_fro while satisfying
    AX == B.
    
    A must be full rank."""

    if not (isinstance(A, (matrix, spmatrix)) and isinstance(B, (matrix, spmatrix))):
        raise TypeError('both arguments must be matrices')

    if rows(A) > cols(A):
        raise DimError('A must be fat')

    if rows(B) != rows(A):
        raise DimError('rows(B) must equal rows(A)')

    # Pad X to make room for the solution.
    X = concatvert(B, zeros(cols(A) - rows(B), cols(B), full=True))

    lapack.gels(matrix(A), X)
    return X

def eig(A):
    """Calculates the eigenvalues and eigenvectors of a real symmetric matrix.

    Returns (eigenvalues, eigenvectors), where the eigenvalues are sorted in
    increasing order."""
    if not isinstance(A, (matrix, spmatrix)):
        raise TypeError('a symmetric matrix is required')

    eigenvalues = zeros(rows(A), 1, full=True)
    eigenvectors = matrix(A)
    lapack.syevr(matrix(A), eigenvalues, jobz='V', Z=eigenvectors)
    return (eigenvalues, eigenvectors)

def svd(A):
    """Computes the singular value decomposition of a matrix.
    
    Returns (U, S, V) where the columns of U are the (truncated) left singular
    vectors, S contains the singular values in decreasing order, and the
    columns of V are the truncated right singular values. If (U, S, V) =
    svd(A) then A == U*diag(S)*transpose(V)."""
    m, n = size(A)
    k = min(m, n)
    S = zeros(k, 1, full=True)
    U = zeros(m, full=True)
    V = zeros(n, full=True)
    lapack.gesdd(matrix(A), S, jobz='A', U=U, Vt=V)
    return (U[:,0:k], S, tp(V[0:k,:]))

def cholsolve(A, B):
    B = matrix(B)
    cholmod.linsolve(A, B)
    return B

def spy(L):
    import pylab

    if isinstance(L, (matrix, spmatrix)):
        pylab.spy(matrix(L))
        return

    n = 0
    for (i, j) in L.keys():
        if i > n:
            n = i

        if j > n:
            n = j

    A = zeros(n+1)

    for (i, j) in L.keys():
        if isnonzero(L[i,j]):
            A[i,j] = 1.0


    pylab.spy(matrix(A))

def cholnum(A):
    # jem: retired code which will be removed soon. Used for testing code
    # generation of cholesky.
    n = rows(A)

    # Cholesky in Python.
    v = [None,]*n
    d = [None,]*n

    L = zeros(n)

    d[0] = A[0,0]

    for i in range(1, n):
        if isnonzero(A[i,0]):
            L[i,0] = A[i,0] / d[0]

    for j in range(1, n):
        for i in range(j):
            if iszero(L[j,i]) or iszero(d[i]):
                v[i] = 0
            else:
                v[i] = L[j,i]*d[i]

        v[j] = A[j,j]
        for i in range(j):
            if isnonzero(L[j,i]*v[i]):
                v[j] -= L[j,i]*v[i]

        d[j] = v[j]

        if j < n - 1:
            for i in range(j + 1, n):
                if iszero(A[i,j]):
                    L[i,j] = 0
                else:
                    L[i,j] = A[i,j]
                for k in range(j):
                    if isnonzero(L[i,k]*v[k]):
                        L[i,j] -= L[i,k]*v[k]
                if isnonzero(L[i,j]):
                    L[i,j] /= v[j]

    # Could replace this later by knowledge that these are identically 1.
    # Need some special token to add here, perhaps?
    for i in range(n):
        L[i,i] = 1

    return (sparse(L), matrix(d))




#!/usr/bin/env python

""" Script for fixing a reaction file

This script constructs a ReactionParser and calls its fixfile() method,
which replaces spaces in reaction or metabolite names with underscores.


This file is part of metano.
Copyright (C) 2010-2014 Alexander Riemer,
Braunschweig University of Technology,
Dept. of Bioinformatics and Biochemistry

metano is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

metano is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with metano.  If not, see <http://www.gnu.org/licenses/>.
"""

from fba import OptionParser
from reactionparser import ReactionParser
from defines import Verbosity, COPYRIGHT_VERSION_STRING
import os


def main():
    # Parse command line

    usage = "Usage: %prog [options]"
    version = "%prog\n" + COPYRIGHT_VERSION_STRING
    parser = OptionParser(usage=usage, version=version)
    parser.add_option("-i", "--input", dest="inputFile", help="use the "
                      "given reaction FILE as input", metavar="FILE")
    parser.add_option("-o", "--output", dest="outputFile", help="write output "
                      "(reaction file) to FILE (must not be the same as input "
                      "file)", metavar="FILE")

    options, _ = parser.parse_args()
    parser.check_required("-i")
    parser.check_required("-o")

    if os.path.exists(options.outputFile):
        if os.path.samefile(options.outputFile, options.inputFile):
            print "Error: Output file must not be the same as input file!"
            exit()

    # Call reaction parser for fixing the reaction file

    rparser = ReactionParser()
    try:
        rparser.fixfile(options.inputFile, options.outputFile)
    except IOError, strerror:
        print ("An error occurred while trying to read file %s or write file "
               "%s:" % (os.path.normpath(options.inputFile),
                        os.path.normpath(options.outputFile)))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("Error in reaction file %s:" %
               os.path.normpath(options.inputFile))
        print strerror
        exit()

    # Print warning and info messages
    print '\n'.join(rparser.getMessages(Verbosity.ALL))


if __name__ == "__main__":
    main()



"""
This module defines the class LinearProblem, which represents a linear problem
to be solved by an appropriate solver.


This file is part of metano.
Copyright (C) 2010-2014 Alexander Riemer,
Braunschweig University of Technology,
Dept. of Bioinformatics and Biochemistry

metano is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

metano is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with metano.  If not, see <http://www.gnu.org/licenses/>.
"""

from openopt import LP
from numpy import dot, inf, isinf
from defines import SolverStatus, glpkToSolverStatus
import pymprog


_GLPK = 'glpk'
_OOGLPK = 'ooglpk'


class SolverError(Exception):
    """ Exception raised if an unknown or inapplicable solver is selected
    """
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)


class LinearProblem(object):

    # Table of linear problem solvers with corresponding OpenOpt/internal name
    solvers = {'default' : 'glpk',
               'lpsolve' : 'lpSolve',
               'cvxopt'  : 'cvxopt_lp',
               'glpk'    : _GLPK,
               'ooglpk'  : _OOGLPK}


    def __init__(self, Aeq, beq=None, Aineq=None, bineq=None, lb=None, ub=None,
                 solver=solvers['default']):
        """ initialize the linear problem

        Keyword arguments:

        Aeq         -- matrix of equality constrains (Aeq * v = beq)
        beq         -- right-hand side vector of equality constraints
        Aineq       -- matrix of inequality constrains (Aineq * v <= bineq)
        bineq       -- right-hand side vector of inequality constraints
        lb          -- list of lower bounds (indexed like matrix columns)
        ub          -- list of upper bounds (indexed like matrix columns)
        solver      -- solver to be used for Linear Programming
        """
        """
        with open("parameters.txt", "w") as file:
            string = str(self) + "\n\n Aeq:\n" + str(Aeq) + "\n\n beq:\n" + str(beq) + "\n\n Aineq:\n" + str(Aineq) + "\n\n bineq:\n" + str(bineq) + "\n\n lb:\n" + str(lb) + "\n\n ub:\n" + str(ub) + "\n\n solver:\n" + str(solver) + "\n"
            file.write(string)
"""
        try:
            self.solver = self.solvers[solver.lower()]
        except KeyError:
            raise SolverError("Unknown solver '%s'. Try one of the following:"
                              "\n%s" % (solver, self.solvers.keys()))
        self.Aeq = Aeq
        self.Aineq = Aineq
        self.bineq = bineq
        self.istop = None
        self.status = SolverStatus.PREPARED

        # Check matrix and lb/ub vectors
        nRows = len(self.Aeq)
        if not nRows:
            raise ValueError("Error: Matrix Aeq has no rows")
        nCols = len(self.Aeq[0])
        if nCols == 0:
            raise ValueError("Error: Matrix Aeq has no columns")

        if lb is None:
            self.lb = [-inf]*nCols
        elif len(lb) != nCols:
            raise ValueError("Error: Vector of lower bounds has wrong length")
        else:
            self.lb = lb

        if ub is None:
            self.ub = [inf]*nCols
        elif len(ub) != nCols:
            raise ValueError("Error: Vector of upper bounds has wrong length")
        else:
            self.ub = ub

        if beq is not None:
            if len(beq) != nRows:
                raise ValueError("Error: Right-hand side vector of equalities "
                                 "has wrong length")
            self.beq = beq
        else:
            # Default: Right-hand side is zero vector with the same number of
            #          rows as matrix
            self.beq = [0.]*nRows

        if Aineq is None:
            nIneq = 0
        else:
            nIneq = len(Aineq)
            if len(bineq) != nIneq:
                raise ValueError("Error: Matrix and vector dimensions disagree "
                                 "in inequality constraints")
            if nIneq > 0 and len(Aineq[0]) != nCols:
                raise ValueError("Error: Matrix Aineq has wrong number of "
                                 "columns.")


        # Result values
        self.obj_value = 0. # value of objective function at solution
        self.solution = []  # solution vector

        if self.solver == _GLPK:
            rangeCols = range(nCols)

            self.glpkP = pymprog.model('LP')
            self.glpkVar = self.glpkP.var('flux', rangeCols)

            # Mass balance and other equality constraints
            self.glpkP.st(sum(self.glpkVar[j] * float(Aeq[i][j])
                              for j in rangeCols if Aeq[i][j] != 0.) ==
                              float(beq[i]) for i in range(nRows))

            # Flux bounds constraints
            self.glpkP.st([self.glpkVar[i] >= float(lb[i])
                           for i in rangeCols if not isinf(lb[i])])
            self.glpkP.st([self.glpkVar[i] <= float(ub[i])
                           for i in rangeCols if not isinf(ub[i])])

            # Inequality constraints
            if nIneq:
                self.glpkP.st(sum(self.glpkVar[j] * float(Aineq[i][j])
                                  for j in rangeCols if Aineq[i][j] != 0.)
                                  <= float(bineq[i]) for i in range(nIneq))

            # Set to verbose
            #self.glpkP.verb=False


    def setInequalityConstraints(self, Aineq=None, bineq=None):
        """ set inequality constraints for optimization (or unset if None)
        """
        nCols = len(self.lb)
        if Aineq is None:
            nIneq = 0
        else:
            nIneq = len(Aineq)
            if nIneq != len(bineq):
                raise ValueError("Error: Matrix and vector dimensions disagree "
                                 "in inequality constraints")
            if nIneq > 0 and len(Aineq[0]) != nCols:
                raise ValueError("Error: Matrix Aineq has wrong number of "
                                 "columns.")
        self.Aineq = Aineq
        self.bineq = bineq

        if self.solver == _GLPK:
            # Construct new model object (because constraints can't be unset)
            rangeCols = range(nCols)

            self.glpkP = pymprog.model('LP')
            self.glpkVar = self.glpkP.var('flux', rangeCols)
            nRows = len(self.Aeq)

            # Mass balance constraints
            self.glpkP.st(sum(self.glpkVar[j] * float(self.Aeq[i][j])
                              for j in rangeCols if self.Aeq[i][j] != 0.) ==
                              float(self.beq[i]) for i in range(nRows))

            # Flux bounds constraints
            self.glpkP.st([self.glpkVar[i] >= float(self.lb[i])
                           for i in rangeCols if not isinf(self.lb[i])])
            self.glpkP.st([self.glpkVar[i] <= float(self.ub[i])
                           for i in rangeCols if not isinf(self.ub[i])])

            if nIneq:
                # Inequality constraints
                self.glpkP.st(sum(self.glpkVar[j] * float(Aineq[i][j])
                                  for j in rangeCols if Aineq[i][j] != 0.)
                                  <= float(bineq[i]) for i in range(nIneq))


    def setObjective(self, obj):
        """ set objective function as coefficient vector

        Keyword arguments:
        obj         -- the linear objective function (coefficients in the linear
                       combination of matrix columns, i.e. fluxes) (negative for
                       maximization, positive for minimization)
        """
        self.obj = obj
        if len(self.obj) != len(self.lb):
            raise ValueError("Error: Objective function vector has wrong "
                             "length")
        if self.solver == _GLPK:
            self.objIndices = [i for i in range(len(obj)) if obj[i] != 0.]
            self.glpkP.min(sum(self.glpkVar[i]*obj[i] for i in self.objIndices),
                           "objective")

    def resetObjective(self):
        self.obj = [0.]*len(self.obj)
        if self.solver == _GLPK:
            self.glpkP.min(sum(self.glpkVar[i]*0. for i in self.objIndices),
                           "objective")


    def minimize(self, **kwargs):
        """ solve the linear problem with the solver given in self.solver

        Returns:
        obj_value, solution

        obj_value -- value of the objective function at the discovered solution
        solution  -- the solution flux vector (indexed like matrix columns)
        """
        if self.solver == _GLPK:
            # Use pyMathProg interface to GLPK solver
            self.glpkP.solve()

            self.istop = self.glpkP.status()
#            print "istop:", self.istop
            self.status = glpkToSolverStatus(self.istop)

            if self.status == SolverStatus.OPTIMAL:
                self.obj_value = self.glpkP.vobj()
                self.solution = [self.glpkVar[i].primal for i in
                                 range(len(self.glpkVar))]
            else:
                self.obj_value = 0.
                self.solution = []
        else:
            # Use OpenOpt interface
            lp = LP(self.obj, A=self.Aineq, Aeq=self.Aeq, b=self.bineq,
                    beq=self.beq, lb=self.lb, ub=self.ub, **kwargs)
            lp.debug=1
#            lp.iprint=-1 # suppress solver output
            r = lp.solve(self.solver if self.solver != _OOGLPK else 'glpk')

            if r.istop <= 0. or r.ff != r.ff:  # check halting condition
                self.obj_value = 0.
                self.solution = []
                self.istop = r.istop
            else:
                self.obj_value = r.ff
                self.solution = r.xf
                self.istop = r.istop
#            print self.istop

        return self.obj_value, self.solution


    def eval(self, v):
        """ evaluate objective function for vector v
        """
        return dot(self.obj, v)



#!/usr/bin/env python

""" This script computes the split ratios for the fluxes entering and leaving
    each metabolite (as percentages).

Usage:

    splitratios.py <solution-file> -r reaction-file [-o output-file]
                   [-m metabolite] [-t threshold] [-u] [-c cutoff] [-f] [-a]


This file is part of metano.
Copyright (C) 2010-2014 Alexander Riemer,
Braunschweig University of Technology,
Dept. of Bioinformatics and Biochemistry

metano is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

metano is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with metano.  If not, see <http://www.gnu.org/licenses/>.
"""

from fba import OptionParser
from reactionparser import ReactionParser
from metabolicmodel import MetabolicModel
from metabolicflux import MetabolicFlux
from defines import COPYRIGHT_VERSION_STRING
import os
import sys


def main():
    # 1. Parse command line

    usage = "Usage: %prog <solution-file> [options]"
    version = "Split-ratio analysis\n" + COPYRIGHT_VERSION_STRING
    parser = OptionParser(usage=usage, version=version)
    parser.add_option("-r", "--reactions", dest="reactionFile", help="use "
                      "metabolic model given by reaction FILE", metavar="FILE")
    parser.add_option("-m", "--metabolite", dest="metabolite", help="NAME of "
                      "the single metabolite to be analyzed (optional)",
                      metavar="NAME")
    parser.add_option("-o", "--output", dest="outputFile", help="perform "
                      "analysis for all metabolites and write output to FILE",
                      metavar="FILE")
    parser.add_option("-t", "--omit-fluxes-below", dest="threshold",
                      type="float", help="omit metabolites in output whose flux"
                      " is below the given THRESHOLD (only with -o)",
                      metavar="THRESHOLD")
    parser.add_option("-u", "--omit-unbranched", dest="omitUnbranched",
                      action="store_true", help="if set, metabolites with "
                      "unbranched flux are omitted (only with -o)")
    parser.add_option("-c", "--cutoff", dest="cutoff", type="float", help="use "
                      "the given cutoff VALUE (default 0)", metavar="VALUE")
    parser.add_option("-f", "--cutoff-is-absolute", dest="cutoffIsAbsolute",
                      action="store_true", help="if set, cutoff is in absolute "
                      "flux units rather than a value between 0 and 1")
    parser.add_option("-a", "--list-all", dest="listAll", action="store_true",
                      help="if set, cutoff is ignored, and even zero fluxes are"
                      " shown (per definition as outgoing)")
    parser.set_defaults(threshold=-1., omitUnBranched=False, cutoff=0.,
                        cutoffIsAbsolute=False, listAll=False)

    options, args = parser.parse_args()
    parser.check_required('-r')

    if not options.metabolite and not options.outputFile:
        print "Neither metabolite nor output file given. Nothing to do."
        exit()

    # 2. Read solution file

    solution = MetabolicFlux()
    try:
        solution.readFromFile(args[0])
    except IndexError:
        print ("Error: No solution file given.\nUsage is\n    " +
               os.path.basename(sys.argv[0]) + " <solution-file> [options]")
        exit()
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(args[0]))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("An error occurred parsing file %s:" %
               os.path.basename(args[0]))
        print strerror
        exit()

    # 3. Parse reaction file

    model = MetabolicModel()
    rparser = ReactionParser()
    try:
        model.addReactionsFromFile(options.reactionFile, rparser)
    except IOError, strerror:
        print ("An error occurred while trying to read file %s:" %
               os.path.basename(options.reactionFile))
        print strerror
        exit()
    except SyntaxError, strerror:
        print ("Error in reaction file %s:" %
               os.path.basename(options.reactionFile))
        print strerror
        exit()

    # 4. Compute split ratios

    if options.outputFile:
        # Compute split ratios for all metabolites
        splitRatios = solution.computeAllSplitRatios(model, options.cutoff,
                        options.cutoffIsAbsolute, options.listAll)
        try:
            writeSplitRatiosToFile(options.outputFile, splitRatios,
                                   options.threshold, options.omitUnbranched)
        except IOError, strerror:
            print ("Unable to write to file %s:" %
                   os.path.basename(options.outputFile))
            print strerror

        # Print statistics
        nMetabolites = len(splitRatios)
        print ("The model contains %u reactions and %u metabolites." %
               (len(model), nMetabolites))
        EPSILON = 1e-6
        nZero = nOne = nBelowEps = nDiff = 0
        maxDiff = 0.
        maxdMet = ""

        for met in splitRatios:
            outRatios, inRatios = splitRatios[met]
            outSum = sum(outRatios[rea][1] for rea in outRatios)
            diffInOut = (abs(outSum - sum(inRatios[rea][1]
                                          for rea in inRatios)))
            if diffInOut > EPSILON:
                nDiff += 1
                if diffInOut > maxDiff:
                    maxDiff = diffInOut
                    maxdMet = met
            if len(outRatios) == 0 or outSum == 0.:
                nZero += 1
            else:
                if len(outRatios) == 1 and len(inRatios) == 1:
                    nOne += 1
                elif options.listAll:
                    # Actually count all outgoing and incoming fluxes > 0
                    nOut = sum(1 for rea in outRatios if outRatios[rea][1] > 0.)
                    nIn  = sum(1 for rea in inRatios if inRatios[rea][1] > 0.)
                    if nOut == nIn == 1:
                        nOne += 1
                if sum(outRatios[rea][1] for rea in outRatios) < EPSILON:
                    nBelowEps += 1

        print ("%u metabolites have a diff. betw. incoming and outgoing "
               "fluxes above %g.\nThe highest diff. betw. incoming and "
               "outgoing metabolite flux is %g (%s)." % (nDiff, EPSILON,
                                                         maxDiff, maxdMet))
        if options.cutoff == 0.:
            print ("%u metabolites (%.2g%%) have flux zero." %
                   (nZero, (float(nZero)/float(nMetabolites))*100.))
        else:
            if options.cutoffIsAbsolute:
                sCutoff = "%g" % options.cutoff
            else:
                sCutoff = "%g%%" % (options.cutoff * 100.)
            print ("%u metabolites (%.2g%%) have no single outgoing flux above "
                   "%s." % (nZero, (float(nZero)/float(nMetabolites))*100.,
                            sCutoff))
        if nBelowEps:
            print ("Another %u metabolites (%.2g%%) have a flux below %g." %
                   (nBelowEps, ((float(nBelowEps)/float(nMetabolites))*100.),
                    EPSILON))
        print ("%u metabolites (%.2g%%) lie on an unbranched path." %
               (nOne, ((float(nOne)/float(nMetabolites))*100.)))
        print

        # Extract data for queried metabolite
        if options.metabolite:
            try:
                outRatios, inRatios = splitRatios[options.metabolite]
            except KeyError:
                print ("Error: Metabolite %r does not exist." %
                       options.metabolite)
                exit()

    elif options.metabolite:
        # Only compute split ratios for the given metabolite
        outRatios, inRatios = solution.computeSplitRatios(options.metabolite,
            model, options.cutoff, options.cutoffIsAbsolute, options.listAll)

    if options.metabolite:
        print "Split ratios for %r" % options.metabolite
        maxlen = max(len(max(outRatios, key=len)), len(max(inRatios, key=len)))
        fluxSum = sum(outRatios[rea][1] for rea in outRatios)
        print "- outgoing (sum %g) -" % fluxSum
        for rea in sorted(outRatios, key=lambda x : outRatios[x][0],
                          reverse=True):
            ratio, flux = outRatios[rea]
            print (rea.ljust(maxlen) + " " + ("%.4g%%" % (ratio*100.)).rjust(10)
                   + " " + ("%.6g" % flux).rjust(12))
        fluxSum = sum(inRatios[rea][1] for rea in inRatios)
        print "- incoming (sum %g) -" % fluxSum
        for rea in sorted(inRatios, key=lambda x : inRatios[x][0],
                          reverse=True):
            ratio, flux = inRatios[rea]
            print (rea.ljust(maxlen) + " " + ("%.4g%%" % (ratio*100.)).rjust(10)
                   + " " + ("%.6g" % flux).rjust(12))


def writeSplitRatiosToFile(filename, splitRatios, threshold, omitUnbranched):
    with open(filename, 'w') as f:
        return writeSplitRatiosToFileHandle(f, splitRatios, threshold,
                                            omitUnbranched)

def writeSplitRatiosToFileHandle(f, splitRatios, threshold, omitUnbranched):
    # sorted[Out/In]Keys stores the sorted keys for the outRatios or inRatios
    #  dict of each metabolite (in descending order of flux)
    sortedOutKeys, sortedInKeys = {}, {}
    # maxwidth stores maximum width of each column
    maxwidth = {}
    metaboliteList = sorted(splitRatios)

    for met in metaboliteList:
        outRatios, inRatios = splitRatios[met]
        sortedOutKeys[met] = sorted(outRatios, key=lambda x : outRatios[x][0],
                                    reverse=True)
        sortedInKeys[met] = sorted(inRatios, key=lambda x : inRatios[x][0],
                                   reverse=True)
        for i in range(len(sortedOutKeys[met])):
            try:
                maxwidth[i] = max(maxwidth[i], len(sortedOutKeys[met][i]))
            except KeyError:
                maxwidth[i] = max(len(sortedOutKeys[met][i]), 12)
        for i in range(len(sortedInKeys[met])):
            try:
                maxwidth[i] = max(maxwidth[i], len(sortedInKeys[met][i]))
            except KeyError:
                maxwidth[i] = max(len(sortedInKeys[met][i]), 12)

    for met in metaboliteList:
        outRatios, inRatios = splitRatios[met]

        if omitUnbranched and len(outRatios) <= 1 and len(inRatios) <=1:
            # Omit metabolite in output if it has an unbranched flux
            continue

        # 1st line: <metabolite name>
        # 2nd line: -outgoing <flux>
        # 3rd line: <rea1> <rea2> ... <reaN>
        # 4th line:  <%1>   <%2>  ...  <%N>
        # 5th line:  <v1>   <v2>  ...  <vN>
        # 6th line: -outgoing <flux>
        # 7th line: <rea1> <rea2> ... <reaN>
        # 8th line:  <%1>   <%2>  ...  <%N>
        # 9th line:  <v1>   <v2>  ...  <vN>
        lines = ["", "", ""]
        fluxSum = 0.
        for i in range(len(sortedOutKeys[met])):
            rea = sortedOutKeys[met][i]
            ratio, flux = outRatios[rea]
            fluxSum += flux
            lines[0] += " " + rea.rjust(maxwidth[i])
            lines[1] += " " + ("%.4g%%" % (ratio*100.)).rjust(maxwidth[i])
            lines[2] += " " + ("%.6g" % flux).rjust(maxwidth[i])

        if fluxSum < threshold:
            # Omit metabolite in output if its total flux is below threshold
            continue

        s = "%s\n-outgoing %g\n" % (met, fluxSum)
        for line in lines:
            if line:
                s += line + '\n'

        lines = ["", "", ""]
        fluxSum = 0.
        for i in range(len(sortedInKeys[met])):
            rea = sortedInKeys[met][i]
            ratio, flux = inRatios[rea]
            fluxSum += flux
            lines[0] += " " + rea.rjust(maxwidth[i])
            lines[1] += " " + ("%.4g%%" % (ratio*100.)).rjust(maxwidth[i])
            lines[2] += " " + ("%.6g" % flux).rjust(maxwidth[i])

        s += "-incoming %g\n" % fluxSum
        for line in lines:
            if line:
                s += line + '\n'
        f.write(s)


if __name__ == "__main__":
    main()
